from collections.abc import Mapping

from panther_core import PantherEvent

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSEC2LaunchUnusualEC2Instances(Rule):
    id = "AWS.EC2.LaunchUnusualEC2Instances-prototype"
    display_name = "AWS EC2 Launch Unusual EC2 Instances"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0002:T1610"]}
    default_description = (
        "Detect when an actor deploys an EC2 instance with an unusual profile based on your business needs.\n"
    )
    default_reference = (
        "https://stratus-red-team.cloud/attack-techniques/AWS/aws.execution.ec2-launch-unusual-instances/\n"
    )
    default_runbook = "Follow up with the instance to identify whether the instance has a legitimate purpose. Reach out to the actor to ensure they performed the action.\n"
    summary_attributes = [
        "p_any_aws_account_ids",
        "p_any_instance_ids",
        "p_any_arns",
        "p_any_aws_tags",
        "p_any_usernames",
    ]
    tags = ["CloudTrail", "EC2", "Execution", "Deploy Container", "Execution:Deploy Container", "Beta"]
    # Configuration Required
    #   Add/remove items from the set below as needed. It should contain instance types which aren't
    #   expected to be used in your environment
    # Large GPU compute, but no graphics - could be used for crypt mining
    UNUSUAL_INSTANCE_TYPES = {"p2.xlarge"}

    def rule(self, event: PantherEvent) -> bool:
        return (
            event.get("eventSource") == "ec2.amazonaws.com"
            and event.get("eventName") == "RunInstances"
            and (self.get_instance_type(event) in self.get_unusual_instance_types())
        )

    def title(self, event: PantherEvent) -> str:
        # The actor in these events is always AutoScalingService
        account = event.get("recipientAccountId")
        instance_type = self.get_instance_type(event)
        return f"EC2 instance with a suspicious type '{instance_type}' was launched in in {account}"

    def severity(self, event: PantherEvent) -> str:
        if not aws_cloudtrail_success(event):
            return "LOW"
        return "DEFAULT"

    def alert_context(self, event: PantherEvent) -> dict:
        context = aws_rule_context(event)
        context["instanceType"] = self.get_instance_type(event)
        return context

    def get_unusual_instance_types(self) -> set[str]:
        # Making this a separate function allows us to mock it during unit tests for reliable testing!
        return self.UNUSUAL_INSTANCE_TYPES

    def get_instance_type(self, event: PantherEvent) -> str:
        # Return the type of the instance that was launch
        instance_type = event.deep_get("requestParameters", "instanceType", default="<UNKNOWN INSTANCE TYPE>")
        # instanceType could be a string or a dict
        if isinstance(instance_type, Mapping):
            instance_type = instance_type.get("value", "<UNKNOWN INSTANCE TYPE>")
        return instance_type

    tests = [
        RuleTest(
            name="Successful Unusual EC2",
            expected_result=True,
            mocks=[RuleMock(object_name="get_unusual_instance_types", return_value="p2.xlarge")],
            log={
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "41fab871-150b-43ad-b42a-39fff3f2ca4e",
                "eventName": "RunInstances",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2024-12-16 18:41:07.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "95cdbe4d-8ff7-4111-8f08-44f510371035",
                "requestParameters": {
                    "availabilityZone": "us-west-2a",
                    "blockDeviceMapping": {},
                    "clientToken": "fleet-180da986-0bb4-c936-0c9a-0e20a0c6d1aa-0",
                    "disableApiStop": False,
                    "disableApiTermination": False,
                    "instanceType": "p2.xlarge",
                    "instancesSet": {"items": [{"maxCount": 1, "minCount": 1}]},
                    "monitoring": {"enabled": False},
                    "subnetId": "subnet-083e5906ef2809ac2",
                },
                "responseElements": {
                    "groupSet": {},
                    "instancesSet": {
                        "items": [
                            {
                                "amiLaunchIndex": 0,
                                "architecture": "arm64",
                                "blockDeviceMapping": {},
                                "bootMode": "uefi",
                                "capacityReservationSpecification": {"capacityReservationPreference": "open"},
                                "clientToken": "fleet-180da986-0bb4-c936-0c9a-0e20a0c6d1aa-0",
                                "cpuOptions": {"coreCount": 2, "threadsPerCore": 1},
                                "currentInstanceBootMode": "uefi",
                                "ebsOptimized": False,
                                "enaSupport": True,
                                "enclaveOptions": {"enabled": False},
                                "groupSet": {"items": [{"groupId": "sg-03d704b35372e74e8", "groupName": "my-group"}]},
                                "hypervisor": "xen",
                                "iamInstanceProfile": {
                                    "arn": "arn:aws:iam::111122223333:instance-profile/profile-id",
                                    "id": "PROFILE_ID",
                                },
                                "imageId": "ami-013e7d3a6659f358d",
                                "instanceId": "i-07d06021b0da55115",
                                "instanceState": {"code": 0, "name": "pending"},
                                "instanceType": "p2.xlarge",
                                "launchTime": 1734374467000,
                                "maintenanceOptions": {"autoRecovery": "default"},
                                "metadataOptions": {
                                    "httpEndpoint": "enabled",
                                    "httpProtocolIpv4": "enabled",
                                    "httpProtocolIpv6": "disabled",
                                    "httpPutResponseHopLimit": 2,
                                    "httpTokens": "required",
                                    "instanceMetadataTags": "disabled",
                                    "state": "pending",
                                },
                                "monitoring": {"state": "disabled"},
                                "networkInterfaceSet": {
                                    "items": [
                                        {
                                            "attachment": {
                                                "attachTime": 1734374467000,
                                                "attachmentId": "eni-attach-022e4a3077e096442",
                                                "deleteOnTermination": True,
                                                "deviceIndex": 0,
                                                "networkCardIndex": 0,
                                                "status": "attaching",
                                            },
                                            "groupSet": {
                                                "items": [
                                                    {
                                                        "groupId": "sg-03d704b35372e74e8",
                                                        "groupName": "eks-cluster-sg-k8s-goat-cluster-816437967",
                                                    },
                                                ],
                                            },
                                            "interfaceType": "interface",
                                            "ipv6AddressesSet": {},
                                            "macAddress": "02:fc:9a:8a:db:c3",
                                            "networkInterfaceId": "eni-03ac9043f76fab96c",
                                            "operator": {"managed": False},
                                            "ownerId": "111122223333",
                                            "privateDnsName": "ip-192-168-1-95.us-west-2.compute.internal",
                                            "privateIpAddress": "192.168.1.95",
                                            "privateIpAddressesSet": {
                                                "item": [
                                                    {
                                                        "primary": True,
                                                        "privateDnsName": "ip-192-168-1-95.us-west-2.compute.internal",
                                                        "privateIpAddress": "192.168.1.95",
                                                    },
                                                ],
                                            },
                                            "sourceDestCheck": True,
                                            "status": "in-use",
                                            "subnetId": "subnet-083e5906ef2809ac2",
                                            "tagSet": {},
                                            "vpcId": "vpc-0330bfd33da75b36e",
                                        },
                                    ],
                                },
                                "operator": {"managed": False},
                                "placement": {"availabilityZone": "us-west-2a", "tenancy": "default"},
                                "privateDnsName": "ip-192-168-1-95.us-west-2.compute.internal",
                                "privateDnsNameOptions": {
                                    "enableResourceNameDnsAAAARecord": False,
                                    "enableResourceNameDnsARecord": False,
                                    "hostnameType": "ip-name",
                                },
                                "privateIpAddress": "192.168.1.95",
                                "productCodes": {},
                                "rootDeviceName": "/dev/xvda",
                                "rootDeviceType": "ebs",
                                "sourceDestCheck": True,
                                "stateReason": {"code": "pending", "message": "pending"},
                                "subnetId": "subnet-083e5906ef2809ac2",
                                "tagSet": {
                                    "items": [
                                        {"key": "k8s.io/cluster-autoscaler/enabled", "value": "true"},
                                        {
                                            "key": "aws:autoscaling:groupName",
                                            "value": "eks-ng-0ca246e9-cac9e862-bfd9-a821-c9fd-9916df5654eb",
                                        },
                                        {
                                            "key": "aws:ec2:fleet-id",
                                            "value": "fleet-180da986-0bb4-c936-0c9a-0e20a0c6d1aa",
                                        },
                                        {"key": "eks:cluster-name", "value": "k8s-goat-cluster"},
                                        {"key": "eks:nodegroup-name", "value": "ng-0ca246e9"},
                                        {"key": "alpha.eksctl.io/nodegroup-name", "value": "ng-0ca246e9"},
                                        {"key": "alpha.eksctl.io/nodegroup-type", "value": "managed"},
                                        {"key": "k8s.io/cluster-autoscaler/k8s-goat-cluster", "value": "owned"},
                                        {"key": "aws:ec2launchtemplate:id", "value": "lt-07a0b5cea4ece8ffd"},
                                        {"key": "aws:ec2launchtemplate:version", "value": "1"},
                                        {"key": "Name", "value": "k8s-goat-cluster-ng-0ca246e9-Node"},
                                        {"key": "kubernetes.io/cluster/k8s-goat-cluster", "value": "owned"},
                                    ],
                                },
                                "virtualizationType": "hvm",
                                "vpcId": "vpc-0330bfd33da75b36e",
                            },
                        ],
                    },
                    "ownerId": "111122223333",
                    "requestId": "95cdbe4d-8ff7-4111-8f08-44f510371035",
                    "requesterId": "414886084714",
                    "reservationId": "r-0ff0b006325a10345",
                },
                "sourceIPAddress": "autoscaling.amazonaws.com",
                "userAgent": "autoscaling.amazonaws.com",
                "userIdentity": {
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/AWSServiceRoleForAutoScaling/AutoScaling",
                    "invokedBy": "autoscaling.amazonaws.com",
                    "principalId": "PRINCIPAL_ID:AutoScaling",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-12-16T18:41:05Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-service-role/autoscaling.amazonaws.com/AWSServiceRoleForAutoScaling",
                            "principalId": "PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "AWSServiceRoleForAutoScaling",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Failed Unusual EC2",
            expected_result=True,
            mocks=[RuleMock(object_name="get_unusual_instance_types", return_value="p2.xlarge")],
            log={
                "awsRegion": "ca-south-3r",
                "errorCode": "Client.UnauthorizedOperation",
                "errorMessage": "You are not authorized to perform this operation. User: arn:aws:sts::751353041310:assumed-role/stratus-red-team-ec2lui-role-idtzskbvtd/aws-go-sdk-1722511821294449000 is not authorized to perform: ec2:RunInstances on resource: arn:aws:ec2:ca-south-3r:751353041310:instance/* because no identity-based policy allows the ec2:RunInstances action. Encoded authorization failure message: T-kSWIRFn32_fxSgyNzoE36avE5lRaRniAjDs-OdhlNgyecEbeTN_dCroUmnEqAbDOrevkgWv8iyUzs0XJxEDlAcgDztlJ-QPNokwAE1JUrWPZcLqpsuM6kK46d5jCUvmzpU_Egq-fML4ed58JHxMdyU4Iz1WGOb6S3W3FB5jghu3JqyDR1B8S8qHryW-e8H1ukHarLt7Ogr4rvYezZ3sf_DNCPDjCGLOSI75x4W0X4Wcl9B9eAuhG-hRbB8KG3e-15CmtpWvw5brndvmrK0sAKwOdcyI47AXNV1DKVLKBNjxwNSQB4knWTX00TASAtGZYroYLyadRTdjZO_CwPGIkcI7wiuAPwSJTrri9xF8zPb5ZJ-Zt4-fQRZoge3sWBFv_wRNOcdGXu8MidJV1ev4CJOpwygM9bO68S_ueU2u_MvKE_zRYrMzTYSMiBKpZGZBDiIZGOGOSzJK8aZ5_F0g5CzhI0IzBxBQh2QFLF0eZe6prRdYEnOZ33EDlaD68PhuyM5xFYzNATqG8UlMtNG7eE1XCMpAmLRAv8ZSnE0PUMrg-Z7RhLyIb3p37VxzKKQHVTdEarNtE22jp38CJ0uRZy5eiNmu-O3JMLeB-AuSYFFoGPtH6h2dH2uV4Fj27vJ4...",
                "eventCategory": "Management",
                "eventID": "1a4debbb-12e9-4bde-b8c7-ea29002bb2a7",
                "eventName": "RunInstances",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2024-08-01T11:30:23Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.09",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "900138736586",
                "requestID": "b663854b-4ebf-4be3-8de0-9c5471904762",
                "requestParameters": {
                    "blockDeviceMapping": {},
                    "clientToken": "5dd59182-3917-421c-9b2c-7c92954b66ee",
                    "disableApiStop": False,
                    "disableApiTermination": False,
                    "instanceType": "p2.xlarge",
                    "instancesSet": {"items": [{"imageId": "ami-aCBbfd13bdb1d1E4b", "maxCount": 10, "minCount": 1}]},
                    "monitoring": {"enabled": False},
                    "subnetId": "subnet-0e540f0c7ffb48ae9",
                },
                "responseElements": None,
                "sourceIPAddress": "06.237.252.245",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "ec2.ca-south-3r.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "stratus-red-team_c8ff220a-7e52-429b-868f-d979123ed2d3",
                "userIdentity": {
                    "accessKeyId": "ASIA9F6MXE9HSYOXYQOS",
                    "accountId": "900138736586",
                    "arn": "arn:aws:sts::900138736586:assumed-role/stratus-red-team-ec2lui-role-idtzskbvtd/aws-go-sdk-1722511821294449000",
                    "principalId": "AROA13YEHY3VAS32TD341:aws-go-sdk-1722511821294449000",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-08-01T11:30:22Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "900138736586",
                            "arn": "arn:aws:iam::900138736586:role/stratus-red-team-ec2lui-role-idtzskbvtd",
                            "principalId": "AROA13YEHY3VAS32TD341",
                            "type": "Role",
                            "userName": "stratus-red-team-ec2lui-role-idtzskbvtd",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Successful Unusual EC2 (Dictionary)",
            expected_result=True,
            mocks=[RuleMock(object_name="get_unusual_instance_types", return_value="p2.xlarge")],
            log={
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "41fab871-150b-43ad-b42a-39fff3f2ca4e",
                "eventName": "RunInstances",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2024-12-16 18:41:07.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "95cdbe4d-8ff7-4111-8f08-44f510371035",
                "requestParameters": {
                    "availabilityZone": "us-west-2a",
                    "blockDeviceMapping": {},
                    "clientToken": "fleet-180da986-0bb4-c936-0c9a-0e20a0c6d1aa-0",
                    "disableApiStop": False,
                    "disableApiTermination": False,
                    "instanceType": {"value": "p2.xlarge"},
                    "instancesSet": {"items": [{"maxCount": 1, "minCount": 1}]},
                    "monitoring": {"enabled": False},
                    "subnetId": "subnet-083e5906ef2809ac2",
                },
                "responseElements": {
                    "groupSet": {},
                    "instancesSet": {
                        "items": [
                            {
                                "amiLaunchIndex": 0,
                                "architecture": "arm64",
                                "blockDeviceMapping": {},
                                "bootMode": "uefi",
                                "capacityReservationSpecification": {"capacityReservationPreference": "open"},
                                "clientToken": "fleet-180da986-0bb4-c936-0c9a-0e20a0c6d1aa-0",
                                "cpuOptions": {"coreCount": 2, "threadsPerCore": 1},
                                "currentInstanceBootMode": "uefi",
                                "ebsOptimized": False,
                                "enaSupport": True,
                                "enclaveOptions": {"enabled": False},
                                "groupSet": {"items": [{"groupId": "sg-03d704b35372e74e8", "groupName": "my-group"}]},
                                "hypervisor": "xen",
                                "iamInstanceProfile": {
                                    "arn": "arn:aws:iam::111122223333:instance-profile/profile-id",
                                    "id": "PROFILE_ID",
                                },
                                "imageId": "ami-013e7d3a6659f358d",
                                "instanceId": "i-07d06021b0da55115",
                                "instanceState": {"code": 0, "name": "pending"},
                                "instanceType": "p2.xlarge",
                                "launchTime": 1734374467000,
                                "maintenanceOptions": {"autoRecovery": "default"},
                                "metadataOptions": {
                                    "httpEndpoint": "enabled",
                                    "httpProtocolIpv4": "enabled",
                                    "httpProtocolIpv6": "disabled",
                                    "httpPutResponseHopLimit": 2,
                                    "httpTokens": "required",
                                    "instanceMetadataTags": "disabled",
                                    "state": "pending",
                                },
                                "monitoring": {"state": "disabled"},
                                "networkInterfaceSet": {
                                    "items": [
                                        {
                                            "attachment": {
                                                "attachTime": 1734374467000,
                                                "attachmentId": "eni-attach-022e4a3077e096442",
                                                "deleteOnTermination": True,
                                                "deviceIndex": 0,
                                                "networkCardIndex": 0,
                                                "status": "attaching",
                                            },
                                            "groupSet": {
                                                "items": [
                                                    {
                                                        "groupId": "sg-03d704b35372e74e8",
                                                        "groupName": "eks-cluster-sg-k8s-goat-cluster-816437967",
                                                    },
                                                ],
                                            },
                                            "interfaceType": "interface",
                                            "ipv6AddressesSet": {},
                                            "macAddress": "02:fc:9a:8a:db:c3",
                                            "networkInterfaceId": "eni-03ac9043f76fab96c",
                                            "operator": {"managed": False},
                                            "ownerId": "111122223333",
                                            "privateDnsName": "ip-192-168-1-95.us-west-2.compute.internal",
                                            "privateIpAddress": "192.168.1.95",
                                            "privateIpAddressesSet": {
                                                "item": [
                                                    {
                                                        "primary": True,
                                                        "privateDnsName": "ip-192-168-1-95.us-west-2.compute.internal",
                                                        "privateIpAddress": "192.168.1.95",
                                                    },
                                                ],
                                            },
                                            "sourceDestCheck": True,
                                            "status": "in-use",
                                            "subnetId": "subnet-083e5906ef2809ac2",
                                            "tagSet": {},
                                            "vpcId": "vpc-0330bfd33da75b36e",
                                        },
                                    ],
                                },
                                "operator": {"managed": False},
                                "placement": {"availabilityZone": "us-west-2a", "tenancy": "default"},
                                "privateDnsName": "ip-192-168-1-95.us-west-2.compute.internal",
                                "privateDnsNameOptions": {
                                    "enableResourceNameDnsAAAARecord": False,
                                    "enableResourceNameDnsARecord": False,
                                    "hostnameType": "ip-name",
                                },
                                "privateIpAddress": "192.168.1.95",
                                "productCodes": {},
                                "rootDeviceName": "/dev/xvda",
                                "rootDeviceType": "ebs",
                                "sourceDestCheck": True,
                                "stateReason": {"code": "pending", "message": "pending"},
                                "subnetId": "subnet-083e5906ef2809ac2",
                                "tagSet": {
                                    "items": [
                                        {"key": "k8s.io/cluster-autoscaler/enabled", "value": "true"},
                                        {
                                            "key": "aws:autoscaling:groupName",
                                            "value": "eks-ng-0ca246e9-cac9e862-bfd9-a821-c9fd-9916df5654eb",
                                        },
                                        {
                                            "key": "aws:ec2:fleet-id",
                                            "value": "fleet-180da986-0bb4-c936-0c9a-0e20a0c6d1aa",
                                        },
                                        {"key": "eks:cluster-name", "value": "k8s-goat-cluster"},
                                        {"key": "eks:nodegroup-name", "value": "ng-0ca246e9"},
                                        {"key": "alpha.eksctl.io/nodegroup-name", "value": "ng-0ca246e9"},
                                        {"key": "alpha.eksctl.io/nodegroup-type", "value": "managed"},
                                        {"key": "k8s.io/cluster-autoscaler/k8s-goat-cluster", "value": "owned"},
                                        {"key": "aws:ec2launchtemplate:id", "value": "lt-07a0b5cea4ece8ffd"},
                                        {"key": "aws:ec2launchtemplate:version", "value": "1"},
                                        {"key": "Name", "value": "k8s-goat-cluster-ng-0ca246e9-Node"},
                                        {"key": "kubernetes.io/cluster/k8s-goat-cluster", "value": "owned"},
                                    ],
                                },
                                "virtualizationType": "hvm",
                                "vpcId": "vpc-0330bfd33da75b36e",
                            },
                        ],
                    },
                    "ownerId": "111122223333",
                    "requestId": "95cdbe4d-8ff7-4111-8f08-44f510371035",
                    "requesterId": "414886084714",
                    "reservationId": "r-0ff0b006325a10345",
                },
                "sourceIPAddress": "autoscaling.amazonaws.com",
                "userAgent": "autoscaling.amazonaws.com",
                "userIdentity": {
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/AWSServiceRoleForAutoScaling/AutoScaling",
                    "invokedBy": "autoscaling.amazonaws.com",
                    "principalId": "PRINCIPAL_ID:AutoScaling",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-12-16T18:41:05Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-service-role/autoscaling.amazonaws.com/AWSServiceRoleForAutoScaling",
                            "principalId": "PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "AWSServiceRoleForAutoScaling",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context
from pypanther.helpers.base import deep_get


@panther_managed
class AWSCloudTrailShortLifecycle(Rule):
    id = "AWS.CloudTrail.ShortLifecycle-prototype"
    display_name = "AWS CloudTrail Retention Lifecycle Too Short"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0005:T1562.008"]}
    default_description = "Detects when an S3 bucket containing CloudTrail logs has been modified to delete data after a short period of time."
    default_reference = (
        "https://stratus-red-team.cloud/attack-techniques/AWS/aws.defense-evasion.cloudtrail-lifecycle-rule/"
    )
    default_runbook = "Verify whether the bucket in question contains CloudTrail data, and if so, why the lifecycle was changed. Potentally add a filter for this bucket to prevent future false positives."
    tags = [
        "AWS",
        "Cloudtrail",
        "Defense Evasion",
        "Impair Defenses",
        "Disable or Modify Cloud Logs",
        "Defense Evasion:Impair Defenses",
        "Security Control",
        "Beta",
    ]
    # Use this to record the names of your S3 buckets that have cloudtrail logs
    #   If a bucket name isn't mentioned here, we still make a best guess as to whether or not it
    #   contains CloudTrail data, but the confidence rating will be lower, and so will the severity
    CLOUDTRAIL_BUCKETS = ("example_cloudtrail_bucket_name",)
    # This is the minimum length fo time CloudTrail logs should remain in an S3 bucket.
    #   We set this to 7 initially, since this is the recommended amount of time logs ingested by
    #   Panther should remain available. You can modify this if you wish.
    CLOUDTRAIL_MINIMUM_STORAGE_PERIOD_DAYS = 7

    def rule(self, event):
        # Only alert for successful PutBucketLifecycle events
        if not (aws_cloudtrail_success(event) and event.get("eventName") == "PutBucketLifecycle"):
            return False
        # Exit out if the bucket doesn't have cloudtrail logs
        #   We check this be either comparing the bucket name to a list of buckets the user knows has
        #   CT logs, or by heuristically looking at the name and guessing whether it likely has CT logs
        bucket_name = event.deep_get("requestParameters", "bucketName")
        if not bucket_name or (
            not self.is_cloudtrail_bucket(bucket_name) and (not self.guess_is_cloudtrail_bucket(bucket_name))
        ):
            return False
        # Don't alert if the Rule status is disabled
        lifecycle = event.deep_get("requestParameters", "LifecycleConfiguration", "Rule")
        if lifecycle.get("Status") != "Enabled":
            return False
        # Alert if the lifecycle period is short
        duration = deep_get(lifecycle, "Expiration", "Days", default=0)
        return duration < self.CLOUDTRAIL_MINIMUM_STORAGE_PERIOD_DAYS

    def title(self, event):
        bucket_name = event.deep_get("requestParameters", "bucketName", default="<UNKNOWN S3 BUCKET>")
        lifecycle = event.deep_get("requestParameters", "LifecycleConfiguration", "Rule")
        duration = deep_get(lifecycle, "Expiration", "Days", default=0)
        rule_id = lifecycle.get("ID", "<UNKNOWN RULE ID>")
        account = event.deep_get("userIdentity", "accountId", default="<UNKNOWN_AWS_ACCOUNT>")
        return f"S3 Bucket {bucket_name} in account {account} has new rule {rule_id} set to delete CloudTrail logs after {duration} day{('s' if duration != 1 else '')}"

    def severity(self, event):
        # Return lower severity if we aren't positive this bucket has cloudtrail logs.
        bucket_name = event.deep_get("requestParameters", "bucketName")
        if not self.is_cloudtrail_bucket(bucket_name):
            return "LOW"
        return "DEFAULT"

    def alert_context(self, event):
        context = aws_rule_context(event)
        # Add name of S3 bucket, Rule ID, and expiration duration to context
        bucket_name = event.deep_get("requestParameters", "bucketName", default="<UNKNOWN S3 BUCKET>")
        lifecycle = event.deep_get("requestParameters", "LifecycleConfiguration", "Rule")
        duration = deep_get(lifecycle, "Expiration", "Days", default=0)
        rule_id = lifecycle.get("ID", "<UNKNOWN RULE ID>")
        context.update({"bucketName": bucket_name, "lifecycleRuleID": rule_id, "lifecycleRuleDurationDays": duration})
        return context

    def is_cloudtrail_bucket(self, bucket_name: str) -> bool:
        """Returns True if the bucket is known to contain CloudTrail logs."""
        return bucket_name in self.CLOUDTRAIL_BUCKETS

    def guess_is_cloudtrail_bucket(self, bucket_name: str) -> bool:
        """Takes a best guess at whether a bucket contains CloudTrail logs or not."""
        # Maybe one day, this check will get more complex
        return "trail" in bucket_name.lower()

    tests = [
        RuleTest(
            name="1-Day Lifecycle Rule on Known CT Bucket",
            expected_result=True,
            mocks=[RuleMock(object_name="is_cloudtrail_bucket", return_value="true")],
            log={
                "p_event_time": "2024-11-25 22:00:58.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-25 22:05:54.357893092",
                "additionalEventData": {
                    "AuthenticationMethod": "AuthHeader",
                    "CipherSuite": "TLS_AES_128_GCM_SHA256",
                    "SignatureVersion": "SigV4",
                    "bytesTransferredIn": 249,
                    "bytesTransferredOut": 0,
                    "x-amz-id-2": "vf6Ehji6uE8ET3EJvRpIQva7eul9KSAUWVlf87sIKBmLQ0HgdGbswZiHYlVvSr1FdP5DiZze4DRZRAFppKpD4A==",
                },
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "e1ea136d-f372-4cd5-be5f-f317fc80214a",
                "eventName": "PutBucketLifecycle",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2024-11-25 22:00:58.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "4XRNRGRFH6RES629",
                "requestParameters": {
                    "Host": "sample-cloudtrail-bucket-name.s3.us-west-2.amazonaws.com",
                    "LifecycleConfiguration": {
                        "Rule": {
                            "Expiration": {"Days": 1},
                            "Filter": {"Prefix": "*"},
                            "ID": "nuke-cloudtrail-logs-after-1-day",
                            "Status": "Enabled",
                        },
                        "xmlns": "http://s3.amazonaws.com/doc/2006-03-01/",
                    },
                    "bucketName": "sample-cloudtrail-bucket-name",
                    "lifecycle": "",
                },
                "resources": [
                    {
                        "accountId": "111122223333",
                        "arn": "arn:aws:s3:::sample-cloudtrail-bucket-name",
                        "type": "AWS::S3::Bucket",
                    },
                ],
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "sample-cloudtrail-bucket-name.s3.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "[sample-user-agent]",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "SAMPLE_PRINCIPAL_ID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-25T16:53:42Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="1-Day Lifecycle Rule on Assumed CT Bucket",
            expected_result=True,
            mocks=[RuleMock(object_name="is_cloudtrail_bucket", return_value="")],
            log={
                "p_event_time": "2024-11-25 22:00:58.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-25 22:05:54.357893092",
                "additionalEventData": {
                    "AuthenticationMethod": "AuthHeader",
                    "CipherSuite": "TLS_AES_128_GCM_SHA256",
                    "SignatureVersion": "SigV4",
                    "bytesTransferredIn": 249,
                    "bytesTransferredOut": 0,
                    "x-amz-id-2": "vf6Ehji6uE8ET3EJvRpIQva7eul9KSAUWVlf87sIKBmLQ0HgdGbswZiHYlVvSr1FdP5DiZze4DRZRAFppKpD4A==",
                },
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "e1ea136d-f372-4cd5-be5f-f317fc80214a",
                "eventName": "PutBucketLifecycle",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2024-11-25 22:00:58.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "4XRNRGRFH6RES629",
                "requestParameters": {
                    "Host": "sample-cloudtrail-bucket-name.s3.us-west-2.amazonaws.com",
                    "LifecycleConfiguration": {
                        "Rule": {
                            "Expiration": {"Days": 1},
                            "Filter": {"Prefix": "*"},
                            "ID": "nuke-cloudtrail-logs-after-1-day",
                            "Status": "Enabled",
                        },
                        "xmlns": "http://s3.amazonaws.com/doc/2006-03-01/",
                    },
                    "bucketName": "sample-cloudtrail-bucket-name",
                    "lifecycle": "",
                },
                "resources": [
                    {
                        "accountId": "111122223333",
                        "arn": "arn:aws:s3:::sample-cloudtrail-bucket-name",
                        "type": "AWS::S3::Bucket",
                    },
                ],
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "sample-cloudtrail-bucket-name.s3.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "[sample-user-agent]",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "SAMPLE_PRINCIPAL_ID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-25T16:53:42Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Disabled 1-Day Lifecycle Rule on Known CT Bucket",
            expected_result=False,
            mocks=[RuleMock(object_name="is_cloudtrail_bucket", return_value="true")],
            log={
                "p_event_time": "2024-11-25 22:00:58.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-25 22:05:54.357893092",
                "additionalEventData": {
                    "AuthenticationMethod": "AuthHeader",
                    "CipherSuite": "TLS_AES_128_GCM_SHA256",
                    "SignatureVersion": "SigV4",
                    "bytesTransferredIn": 249,
                    "bytesTransferredOut": 0,
                    "x-amz-id-2": "vf6Ehji6uE8ET3EJvRpIQva7eul9KSAUWVlf87sIKBmLQ0HgdGbswZiHYlVvSr1FdP5DiZze4DRZRAFppKpD4A==",
                },
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "e1ea136d-f372-4cd5-be5f-f317fc80214a",
                "eventName": "PutBucketLifecycle",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2024-11-25 22:00:58.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "4XRNRGRFH6RES629",
                "requestParameters": {
                    "Host": "sample-cloudtrail-bucket-name.s3.us-west-2.amazonaws.com",
                    "LifecycleConfiguration": {
                        "Rule": {
                            "Expiration": {"Days": 1},
                            "Filter": {"Prefix": "*"},
                            "ID": "nuke-cloudtrail-logs-after-1-day",
                            "Status": "Disabled",
                        },
                        "xmlns": "http://s3.amazonaws.com/doc/2006-03-01/",
                    },
                    "bucketName": "sample-cloudtrail-bucket-name",
                    "lifecycle": "",
                },
                "resources": [
                    {
                        "accountId": "111122223333",
                        "arn": "arn:aws:s3:::sample-cloudtrail-bucket-name",
                        "type": "AWS::S3::Bucket",
                    },
                ],
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "sample-cloudtrail-bucket-name.s3.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "[sample-user-agent]",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "SAMPLE_PRINCIPAL_ID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-25T16:53:42Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="730-Day Lifecycle Rule on Known CT Bucket",
            expected_result=False,
            mocks=[RuleMock(object_name="is_cloudtrail_bucket", return_value="true")],
            log={
                "p_event_time": "2024-11-26 17:26:06.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-26 17:30:54.113261939",
                "additionalEventData": {
                    "AuthenticationMethod": "AuthHeader",
                    "CipherSuite": "TLS_AES_128_GCM_SHA256",
                    "SignatureVersion": "SigV4",
                    "bytesTransferredIn": 309,
                    "bytesTransferredOut": 0,
                    "x-amz-id-2": "xdjFGuP5MOmtnO6PCaHFNtvmnmUjGLngYLZlKRtdDAihd76he3U1M1QVXbs0q5vZr4Pv7ipRNUU=",
                },
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "a8cfbde4-3b77-430a-b2f3-388d5bb75eb3",
                "eventName": "PutBucketLifecycle",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2024-11-26 17:26:06.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "HAKZ6Z7PDPHET3TQ",
                "requestParameters": {
                    "Host": "s3.us-west-2.amazonaws.com",
                    "LifecycleConfiguration": {
                        "Rule": {
                            "Expiration": {"Days": 730},
                            "Filter": "",
                            "ID": "nuke-cloudtrail-logs-after-730-days",
                            "NoncurrentVersionExpiration": {"NoncurrentDays": 730},
                            "Status": "Enabled",
                        },
                        "xmlns": "http://s3.amazonaws.com/doc/2006-03-01/",
                    },
                    "bucketName": "sample-cloudtrail-bucket-name",
                    "lifecycle": "",
                },
                "resources": [
                    {
                        "accountId": "111122223333",
                        "arn": "arn:aws:s3:::sample-cloudtrail-bucket-name",
                        "type": "AWS::S3::Bucket",
                    },
                ],
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "s3.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "[Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36]",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "SAMPLE_PRINCIPAL_ID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-26T17:23:25Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Completely Unrelated Event",
            expected_result=False,
            log={
                "p_event_time": "2024-11-26 17:23:59.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-26 17:30:54.112906272",
                "additionalEventData": {
                    "AuthenticationMethod": "AuthHeader",
                    "CipherSuite": "TLS_AES_128_GCM_SHA256",
                    "SignatureVersion": "SigV4",
                    "bytesTransferredIn": 0,
                    "bytesTransferredOut": 313,
                    "x-amz-id-2": "CaKGcLO+fHGAWCSQD7+2dEACPcs+Az44FEQT3c5iu+YlJ8sFA++rPcYTr5xGx5/iwaxNWzWWaWQ=",
                },
                "awsRegion": "us-west-2",
                "errorCode": "NoSuchLifecycleConfiguration",
                "errorMessage": "The lifecycle configuration does not exist",
                "eventCategory": "Management",
                "eventID": "41fd8553-6e3c-4942-ad03-aba324ec109e",
                "eventName": "GetBucketLifecycle",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2024-11-26 17:23:59.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "111122223333",
                "requestID": "BTYKZ7VY1EKRSMZM",
                "requestParameters": {
                    "Host": "s3.us-west-2.amazonaws.com",
                    "bucketName": "sample-cloudtrail-bucket-name",
                    "lifecycle": "",
                },
                "resources": [
                    {
                        "accountId": "111122223333",
                        "arn": "arn:aws:s3:::sample-cloudtrail-bucket-name",
                        "type": "AWS::S3::Bucket",
                    },
                ],
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "s3.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "[Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36]",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "SAMPLE_PRINCIPAL_ID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-26T17:23:25Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteRule(Rule):
    id = "GSuite.Rule-prototype"
    display_name = "GSuite Passthrough Rule Triggered"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.INFO
    default_description = "A GSuite rule was triggered.\n"
    default_reference = "https://support.google.com/a/answer/9420866"
    default_runbook = "Investigate what triggered the rule.\n"
    summary_attributes = ["actor:email"]

    def rule(self, event):
        if event.deep_get("id", "applicationName") != "rules":
            return False
        if not event.deep_get("parameters", "triggered_actions"):
            return False
        return True

    def title(self, event):
        rule_severity = event.deep_get("parameters", "severity")
        if event.deep_get("parameters", "rule_name"):
            return "GSuite " + rule_severity + " Severity Rule Triggered: " + event.deep_get("parameters", "rule_name")
        return "GSuite " + rule_severity + " Severity Rule Triggered"

    def severity(self, event):
        return event.deep_get("parameters", "severity", default="INFO")

    tests = [
        RuleTest(
            name="Non Triggered Rule",
            expected_result=False,
            log={
                "id": {"applicationName": "rules"},
                "actor": {"email": "some.user@somedomain.com"},
                "parameters": {"severity": "HIGH", "triggered_actions": None},
            },
        ),
        RuleTest(
            name="High Severity Rule",
            expected_result=True,
            log={
                "id": {"applicationName": "rules"},
                "actor": {"email": "some.user@somedomain.com"},
                "parameters": {
                    "data_source": "DRIVE",
                    "severity": "HIGH",
                    "triggered_actions": [{"action_type": "DRIVE_UNFLAG_DOCUMENT"}],
                },
            },
        ),
        RuleTest(
            name="Medium Severity Rule",
            expected_result=True,
            log={
                "id": {"applicationName": "rules"},
                "actor": {"email": "some.user@somedomain.com"},
                "parameters": {
                    "data_source": "DRIVE",
                    "severity": "MEDIUM",
                    "triggered_actions": [{"action_type": "DRIVE_UNFLAG_DOCUMENT"}],
                },
            },
        ),
        RuleTest(
            name="Low Severity Rule",
            expected_result=True,
            log={
                "id": {"applicationName": "rules"},
                "actor": {"email": "some.user@somedomain.com"},
                "parameters": {"severity": "LOW", "triggered_actions": [{"action_type": "DRIVE_UNFLAG_DOCUMENT"}]},
            },
        ),
        RuleTest(
            name="High Severity Rule with Rule Name",
            expected_result=True,
            log={
                "id": {"applicationName": "rules"},
                "actor": {"email": "some.user@somedomain.com"},
                "parameters": {
                    "severity": "HIGH",
                    "rule_name": "CEO Impersonation",
                    "triggered_actions": [{"action_type": "MAIL_MARK_AS_PHISHING"}],
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteDocOwnershipTransfer(Rule):
    id = "GSuite.DocOwnershipTransfer-prototype"
    display_name = "GSuite Document External Ownership Transfer"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Collection:Data from Information Repositories"]
    reports = {"MITRE ATT&CK": ["TA0009:T1213"]}
    default_severity = Severity.LOW
    default_description = "A GSuite document's ownership was transferred to an external party.\n"
    default_reference = (
        "https://support.google.com/drive/answer/2494892?hl=en&co=GENIE.Platform%3DDesktop&sjid=864417124752637253-EU"
    )
    default_runbook = "Verify that this document did not contain sensitive or private company information.\n"
    summary_attributes = ["actor:email"]

    def rule(self, event):
        if event.get("name") != "change_owner":
            return False
        if event.deep_get("parameters", "visibility") in (
            "shared_internally",
            "people_within_domain_with_link",
            "private",
        ):
            return False
        previous_owner = event.deep_get("parameters", "owner", default="<UNKNOWN USER>")
        new_owner = event.deep_get("parameters", "new_owner", default="<UNKNOWN USER>")
        previous_owner_domain = previous_owner.split("@")[1] if "@" in previous_owner else None
        new_owner_domain = new_owner.split("@")[1] if "@" in new_owner else None
        if previous_owner_domain is None or new_owner_domain is None:
            return False
        if previous_owner_domain != new_owner_domain:
            return True
        return False

    def title(self, event):
        actor = event.deep_get("actor", "email", default="<UNKNOWN USER>")
        previous_owner = event.deep_get("parameters", "owner", default="<UNKNOWN USER>")
        new_owner = event.deep_get("parameters", "new_owner", default="<UNKNOWN USER>")
        return f"User [{actor}] transferred document ownership from [{previous_owner}] to [{new_owner}]"

    tests = [
        RuleTest(
            name="Ownership Transferred Within Organization",
            expected_result=False,
            log={
                "actor": {"email": "alice@panther.com", "profileId": "1234567890"},
                "id": {
                    "applicationName": "drive",
                    "customerId": "C123abcde",
                    "time": "2025-08-12 18:41:56.232000000",
                    "uniqueQualifier": "1234567890",
                },
                "ipAddress": "1.2.3.4",
                "kind": "admin#reports#activity",
                "name": "change_owner",
                "parameters": {
                    "billable": True,
                    "doc_id": "1234567890",
                    "doc_title": "sensitive_document.xlsx",
                    "doc_type": "msexcel",
                    "new_owner": "bob@panther.com",
                    "owner": "alice@panther.com",
                    "primary_event": True,
                    "visibility": "shared_internally",
                },
                "type": "acl_change",
            },
        ),
        RuleTest(
            name="Document Transferred to External User",
            expected_result=True,
            log={
                "actor": {"email": "alice@panther.com", "profileId": "1234567890"},
                "id": {
                    "applicationName": "drive",
                    "customerId": "C123abcde",
                    "time": "2025-07-11 19:50:09.324000000",
                    "uniqueQualifier": "1234567890",
                },
                "kind": "admin#reports#activity",
                "name": "change_owner",
                "parameters": {
                    "billable": True,
                    "doc_id": "1234567890",
                    "doc_title": "sensitive_document.xlsx",
                    "doc_type": "msexcel",
                    "new_owner": "bob@example.com",
                    "owner": "alice@panther.com",
                    "primary_event": True,
                    "visibility": "unknown",
                },
                "type": "acl_change",
            },
        ),
        RuleTest(
            name="Document Transferred to Group within Organization",
            expected_result=False,
            log={
                "actor": {"email": "alice@panther.com", "profileId": "100606748236723241722"},
                "id": {
                    "applicationName": "drive",
                    "customerId": "C123abcde",
                    "time": "2025-07-07 23:50:22.743000000",
                    "uniqueQualifier": "1234567890",
                },
                "ipAddress": "1.2.3.4",
                "kind": "admin#reports#activity",
                "name": "change_owner",
                "parameters": {
                    "billable": True,
                    "doc_id": "1234567890",
                    "doc_title": "sensitive_document.xlsx",
                    "doc_type": "msexcel",
                    "new_owner": "Execs",
                    "new_owner_is_team_drive": True,
                    "new_owner_team_drive_id": "1234567890",
                    "originating_app_id": "1234567890",
                    "owner": "Engineering",
                    "owner_is_shared_drive": True,
                    "owner_is_team_drive": True,
                    "owner_team_drive_id": "1234567890",
                    "primary_event": True,
                    "shared_drive_id": "1234567890",
                    "team_drive_id": "1234567890",
                    "visibility": "shared_internally",
                },
                "type": "acl_change",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import deep_get


@panther_managed
class GCPGCSPublic(Rule):
    id = "GCP.GCS.Public-prototype"
    display_name = "GCS Bucket Made Public"
    dedup_period_minutes = 15
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Google Cloud Storage", "Collection:Data From Cloud Storage Object"]
    reports = {"MITRE ATT&CK": ["TA0009:T1530"]}
    default_severity = Severity.HIGH
    default_description = "Adversaries may access data objects from improperly secured cloud storage."
    default_runbook = "Validate the GCS bucket change was safe."
    default_reference = "https://cloud.google.com/storage/docs/access-control/making-data-public"
    summary_attributes = ["severity", "p_any_ip_addresses", "p_any_domain_names"]
    GCS_READ_ROLES = {"roles/storage.objectAdmin", "roles/storage.objectViewer", "roles/storage.admin"}
    GLOBAL_USERS = {"allUsers", "allAuthenticatedUsers"}

    def rule(self, event):
        if event.deep_get("protoPayload", "methodName") != "storage.setIamPermissions":
            return False
        service_data = event.deep_get("protoPayload", "serviceData")
        if not service_data:
            return False
        # Reference: https://cloud.google.com/iam/docs/policies
        binding_deltas = deep_get(service_data, "policyDelta", "bindingDeltas")
        if not binding_deltas:
            return False
        for delta in binding_deltas:
            if delta.get("action") != "ADD":
                continue
            if delta.get("member") in self.GLOBAL_USERS and delta.get("role") in self.GCS_READ_ROLES:
                return True
        return False

    def title(self, event):
        return f"GCS bucket [{event.deep_get('resource', 'labels', 'bucket_name', default='<UNKNOWN_BUCKET>')}] made public"

    tests = [
        RuleTest(
            name="GCS AllUsers Read Permission",
            expected_result=True,
            log={
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "status": {},
                    "authenticationInfo": {"principalEmail": "user.name@runpanther.io"},
                    "requestMetadata": {
                        "callerIp": "136.24.229.58",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe)",
                        "requestAttributes": {"time": "2020-05-15T04:28:42.243082428Z", "auth": {}},
                        "destinationAttributes": {},
                    },
                    "serviceName": "storage.googleapis.com",
                    "methodName": "storage.setIamPermissions",
                    "authorizationInfo": [
                        {
                            "resource": "projects/_/buckets/jacks-test-bucket",
                            "permission": "storage.buckets.setIamPolicy",
                            "granted": True,
                            "resourceAttributes": {},
                        },
                    ],
                    "resourceName": "projects/_/buckets/jacks-test-bucket",
                    "serviceData": {
                        "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                        "policyDelta": {
                            "bindingDeltas": [
                                {"action": "ADD", "role": "roles/storage.objectViewer", "member": "allUsers"},
                            ],
                        },
                    },
                    "resourceLocation": {"currentLocations": ["us"]},
                },
                "insertId": "15cp9rve72xt1",
                "resource": {
                    "type": "gcs_bucket",
                    "labels": {
                        "bucket_name": "jacks-test-bucket",
                        "project_id": "western-verve-123456",
                        "location": "us",
                    },
                },
                "timestamp": "2020-05-15T04:28:42.237027213Z",
                "severity": "NOTICE",
                "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Factivity",
                "receiveTimestamp": "2020-05-15T04:28:42.900626148Z",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.cloudflare import cloudflare_fw_alert_context


@panther_managed
class CloudflareFirewallL7DDoS(Rule):
    id = "Cloudflare.Firewall.L7DDoS-prototype"
    display_name = "Cloudflare L7 DDoS"
    log_types = [LogType.CLOUDFLARE_FIREWALL]
    tags = ["Cloudflare", "Variable Severity"]
    default_severity = Severity.MEDIUM
    default_description = "Layer 7 Distributed Denial of Service (DDoS) detected"
    default_runbook = "Inspect and monitor internet-facing services for potential outages"
    default_reference = "https://www.cloudflare.com/en-gb/learning/ddos/application-layer-ddos-attack/"
    threshold = 100
    summary_attributes = ["Action", "ClientCountry", "ClientIP", "ClientRequestUserAgent"]

    def rule(self, event):
        return event.get("Source", "") == "l7ddos"

    def title(self, _):
        return "Cloudflare: Detected L7 DDoS"

    def alert_context(self, event):
        return cloudflare_fw_alert_context(event)

    def severity(self, event):
        if event.get("Action", "") == "block":
            return "Info"
        return "Medium"

    tests = [
        RuleTest(
            name="Traffic Marked as L7DDoS",
            expected_result=True,
            log={
                "Action": "skip",
                "ClientASN": 55836,
                "ClientASNDescription": "RELIANCEJIO-IN Reliance Jio Infocomm Limited",
                "ClientCountry": "in",
                "ClientIP": "127.0.0.1",
                "ClientRequestHost": "example.com",
                "ClientRequestMethod": "GET",
                "ClientRequestPath": "/main.php",
                "ClientRequestProtocol": "HTTP/1.1",
                "ClientRequestQuery": "",
                "ClientRequestScheme": "http",
                "ClientRequestUserAgent": "Fuzz Faster U Fool v1.3.1-dev",
                "Datetime": "2022-05-10 06:36:57",
                "EdgeColoCode": "DEL",
                "EdgeResponseStatus": 403,
                "Kind": "firewall",
                "MatchIndex": 0,
                "Metadata": {"dos-source": "dosd-edge"},
                "OriginResponseStatus": 0,
                "OriginatorRayID": "00",
                "RayID": "7090a9da88e333d8",
                "RuleID": "ed651449c4a54f4b99c6e3bf863134d5",
                "Source": "l7ddos",
            },
        ),
        RuleTest(
            name="Traffic Marked as L7DDoS but blocked ( INFO level alert )",
            expected_result=True,
            log={
                "Action": "block",
                "ClientASN": 55836,
                "ClientASNDescription": "RELIANCEJIO-IN Reliance Jio Infocomm Limited",
                "ClientCountry": "in",
                "ClientIP": "127.0.0.1",
                "ClientRequestHost": "example.com",
                "ClientRequestMethod": "GET",
                "ClientRequestPath": "/main.php",
                "ClientRequestProtocol": "HTTP/1.1",
                "ClientRequestQuery": "",
                "ClientRequestScheme": "http",
                "ClientRequestUserAgent": "Fuzz Faster U Fool v1.3.1-dev",
                "Datetime": "2022-05-10 06:36:57",
                "EdgeColoCode": "DEL",
                "EdgeResponseStatus": 403,
                "Kind": "firewall",
                "MatchIndex": 0,
                "Metadata": {"dos-source": "dosd-edge"},
                "OriginResponseStatus": 0,
                "OriginatorRayID": "00",
                "RayID": "7090a9da88e333d8",
                "RuleID": "ed651449c4a54f4b99c6e3bf863134d5",
                "Source": "l7ddos",
            },
        ),
        RuleTest(
            name="Traffic Not Marked as L7DDoS",
            expected_result=False,
            log={
                "Action": "block",
                "ClientASN": 55836,
                "ClientASNDescription": "RELIANCEJIO-IN Reliance Jio Infocomm Limited",
                "ClientCountry": "in",
                "ClientIP": "127.0.0.1",
                "ClientRequestHost": "example.com",
                "ClientRequestMethod": "GET",
                "ClientRequestPath": "/main.php",
                "ClientRequestProtocol": "HTTP/1.1",
                "ClientRequestQuery": "",
                "ClientRequestScheme": "http",
                "ClientRequestUserAgent": "Mozilla/5.0 (Windows NT 6.4; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2225.0 Safari/537.36",
                "Datetime": "2022-05-10 06:36:57",
                "EdgeColoCode": "DEL",
                "EdgeResponseStatus": 403,
                "Kind": "firewall",
                "MatchIndex": 0,
                "Metadata": {"dos-source": "dosd-edge"},
                "OriginResponseStatus": 0,
                "OriginatorRayID": "00",
                "RayID": "708174c00f61faa8",
                "RuleID": "e35c9a670b864a3ba0203ffb1bc977d1",
                "Source": "firewallmanaged",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class AWSEC2Monitoring(Rule):
    default_description = "Checks CloudTrail for occurrences of EC2 Image Actions."
    display_name = "AWS EC2 Image Monitoring"
    reports = {"MITRE ATT&CK": ["TA0002:T1204"]}
    default_runbook = "Verify that the action was not taken by a malicious actor."
    default_reference = "https://docs.aws.amazon.com/service-authorization/latest/reference/list_amazonec2imagebuilder.html#amazonec2imagebuilder-actions-as-permissions"
    default_severity = Severity.INFO
    tags = ["ec2"]
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.EC2.Monitoring-prototype"
    # AWS CloudTrail API eventNames for EC2 Image Actions
    EC2_IMAGE_ACTIONS = [
        "CopyFpgaImage",
        "CopyImage",
        "CreateFpgaImage",
        "CreateImage",
        "CreateRestoreImageTask",
        "CreateStoreImageTask",
        "ImportImage",
    ]

    def rule(self, event):
        # Disqualify any eventSource that is not ec2
        if event.get("eventSource", "") != "ec2.amazonaws.com":
            return False
        # Disqualify AWS Service-Service operations, which can appear in a variety of forms
        if (
            event.get("sourceIPAddress", "").endswith(".amazonaws.com")
            or event.deep_get("userIdentity", "type", default="") == "AWSService"
            or event.deep_get("userIdentity", "invokedBy", default="") == "AWS Internal"
            or event.deep_get("userIdentity", "invokedBy", default="").endswith(".amazonaws.com")
        ):
            # FYI there is a weird quirk in the sourceIPAddress field of CloudTrail
            #  events with ec2.amazonaws.com as the source name where users of the
            #  web-console will have their sourceIPAddress recorded as "AWS Internal"
            #  though their userIdentity will be more normal.
            #  Example cloudtrail event in the "Terminate instance From WebUI with assumedRole" test
            return False
        # Dry run operations get logged as SES Internal in the sourceIPAddress
        #  but not in the invokedBy field
        if event.get("errorCode", "") == "Client.DryRunOperation":
            return False
        # Disqualify any eventNames that do not Include Image Actions
        # and events that have readOnly set to false
        if event.get("eventName", "") in self.EC2_IMAGE_ACTIONS:
            return True
        return False

    def title(self, event):
        return f"[{event.deep_get('userIdentity', 'sessionContext', 'sessionIssuer', 'userName')}] triggered a CloudTrail action [{event.get('eventName')}] within AWS Account ID: [{event.get('recipientAccountId')}]"

    tests = [
        RuleTest(
            name="CopyImage",
            expected_result=True,
            log={
                "awsRegion": "us-east-1",
                "eventCategory": "Management",
                "eventID": "0ea3f05a-066c-43f9-8869-393ba67e7936",
                "eventName": "CreateImage",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2022-09-29 22:25:17",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_any_aws_account_ids": ["123456789101"],
                "p_any_aws_arns": [
                    "arn:aws:iam::123456789101:role/DevAdministrator",
                    "arn:aws:sts::123456789101:assumed-role/DevAdministrator/test_user",
                ],
                "p_any_aws_instance_ids": ["i-0381a3817f72a949d"],
                "p_any_domain_names": ["AWS Internal"],
                "p_any_trace_ids": ["ASIA5PZQZ5QHE2FUNXHR"],
                "p_event_time": "2022-09-29 22:25:17",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-09-29 22:27:25.748",
                "p_row_id": "66011977ec1fd0cf9dacf7d913f08d06",
                "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
                "p_source_label": "CloudTrail Logs",
                "readOnly": False,
                "recipientAccountId": "123456789101",
                "requestID": "e686939a-a08a-4fd6-abf5-9ea34793cf25",
                "requestParameters": {
                    "blockDeviceMapping": {
                        "items": [{"deviceName": "/dev/xvda", "ebs": {"deleteOnTermination": True, "volumeSize": 8}}],
                    },
                    "instanceId": "i-0381a3817f72a949d",
                    "name": "testimage",
                    "noReboot": False,
                },
                "responseElements": {
                    "imageId": "ami-06aaf5e4b77161786",
                    "requestId": "e686939a-a08a-4fd6-abf5-9ea34793cf25",
                },
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "AWS Internal",
                "userAgent": "AWS Internal",
                "userIdentity": {
                    "accessKeyId": "ASIA5PZQZ5QHE2FUNXHR",
                    "accountId": "123456789101",
                    "arn": "arn:aws:sts::123456789101:assumed-role/DevAdministrator/test_user",
                    "principalId": "AROA5PZQZ5QHBULW27VAC:test_user",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-09-29T22:22:46Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789101",
                            "arn": "arn:aws:iam::123456789101:role/DevAdministrator",
                            "principalId": "AROA5PZQZ5QHBULW27VAC",
                            "type": "Role",
                            "userName": "DevAdministrator",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="RunInstance",
            expected_result=False,
            log={
                "awsRegion": "us-east-1",
                "eventCategory": "Management",
                "eventID": "015c585b-cbc2-4f9e-9c52-a2f22f3c09f4",
                "eventName": "RunInstances",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2022-10-20 14:16:43",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_any_aws_account_ids": ["123123123123"],
                "p_any_aws_arns": [
                    "arn:aws:iam::123123123123:role/DevAdministrator",
                    "arn:aws:sts::123123123123:assumed-role/DevAdministrator/temp_user",
                ],
                "p_any_aws_instance_ids": ["i-0d7e4b9be8a0de6ea"],
                "p_any_aws_tags": ["Name:test2"],
                "p_any_domain_names": ["AWS Internal", "ip-111.111.111.111.ec2.internal"],
                "p_any_ip_addresses": ["111.111.111.111"],
                "p_any_trace_ids": ["ASIA5PZQZ5QHAW6RFPO5"],
                "p_any_usernames": ["DevAdministrator"],
                "p_event_time": "2022-10-20 14:16:43",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-10-20 14:21:06.845",
                "p_row_id": "be7b3bed9bb891e4a8b9f38d149909",
                "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
                "p_source_label": "Panther Identity Org CloudTrail",
                "readOnly": False,
                "recipientAccountId": "123123123123",
                "requestID": "25c95576-a825-44fd-971d-5a52c1e3b2be",
                "requestParameters": {
                    "blockDeviceMapping": {},
                    "disableApiStop": False,
                    "disableApiTermination": False,
                    "ebsOptimized": False,
                    "instanceType": "t1.micro",
                    "instancesSet": {
                        "items": [{"imageId": "ami-026b57f3c383c2eec", "keyName": "kp1", "maxCount": 1, "minCount": 1}],
                    },
                    "monitoring": {"enabled": False},
                    "networkInterfaceSet": {
                        "items": [
                            {
                                "associatePublicIpAddress": True,
                                "deviceIndex": 0,
                                "groupSet": {"items": [{"groupId": "sg-0aebfa21f302bded9"}]},
                            },
                        ],
                    },
                    "privateDnsNameOptions": {
                        "enableResourceNameDnsAAAARecord": False,
                        "enableResourceNameDnsARecord": True,
                        "hostnameType": "ip-name",
                    },
                    "tagSpecificationSet": {
                        "items": [{"resourceType": "instance", "tags": [{"key": "Name", "value": "test2"}]}],
                    },
                },
                "responseElements": {
                    "groupSet": {},
                    "instancesSet": {
                        "items": [
                            {
                                "amiLaunchIndex": 0,
                                "architecture": "x86_64",
                                "blockDeviceMapping": {},
                                "capacityReservationSpecification": {"capacityReservationPreference": "open"},
                                "cpuOptions": {"coreCount": 1, "threadsPerCore": 1},
                                "currentInstanceBootMode": "bios",
                                "ebsOptimized": False,
                                "enaSupport": True,
                                "enclaveOptions": {"enabled": False},
                                "groupSet": {
                                    "items": [{"groupId": "sg-0aebfa21f302bded9", "groupName": "launch-wizard-4"}],
                                },
                                "hypervisor": "xen",
                                "imageId": "ami-026b57f3c383c2eec",
                                "instanceId": "i-0d7e4b9be8a0de6ea",
                                "instanceState": {"code": 0, "name": "pending"},
                                "instanceType": "t1.micro",
                                "keyName": "kp1",
                                "launchTime": 1666275403000.0,
                                "maintenanceOptions": {"autoRecovery": "default"},
                                "metadataOptions": {
                                    "httpEndpoint": "enabled",
                                    "httpProtocolIpv4": "enabled",
                                    "httpProtocolIpv6": "disabled",
                                    "httpPutResponseHopLimit": 1,
                                    "httpTokens": "optional",
                                    "instanceMetadataTags": "disabled",
                                    "state": "pending",
                                },
                                "monitoring": {"state": "disabled"},
                                "networkInterfaceSet": {
                                    "items": [
                                        {
                                            "attachment": {
                                                "attachTime": 1666275403000.0,
                                                "attachmentId": "eni-attach-0f01abecb268392c1",
                                                "deleteOnTermination": True,
                                                "deviceIndex": 0,
                                                "networkCardIndex": 0,
                                                "status": "attaching",
                                            },
                                            "groupSet": {
                                                "items": [
                                                    {"groupId": "sg-0aebfa21f302bded9", "groupName": "launch-wizard-4"},
                                                ],
                                            },
                                            "interfaceType": "interface",
                                            "ipv6AddressesSet": {},
                                            "macAddress": "0a:47:b4:21:fe:8d",
                                            "networkInterfaceId": "eni-08b298299ee7c922c",
                                            "ownerId": "123123123123",
                                            "privateDnsName": "ip-111.111.111.111.ec2.internal",
                                            "privateIpAddress": "111.111.111.111",
                                            "privateIpAddressesSet": {
                                                "item": [
                                                    {
                                                        "primary": True,
                                                        "privateDnsName": "ip-111.111.111.111.ec2.internal",
                                                        "privateIpAddress": "111.111.111.111",
                                                    },
                                                ],
                                            },
                                            "sourceDestCheck": True,
                                            "status": "in-use",
                                            "subnetId": "subnet-0ae6d533cb0b18193",
                                            "tagSet": {},
                                            "vpcId": "vpc-0f59e8f1222b0de6a",
                                        },
                                    ],
                                },
                                "placement": {"availabilityZone": "us-east-1a", "tenancy": "default"},
                                "privateDnsName": "ip-111.111.111.111.ec2.internal",
                                "privateDnsNameOptions": {
                                    "enableResourceNameDnsAAAARecord": False,
                                    "enableResourceNameDnsARecord": True,
                                    "hostnameType": "ip-name",
                                },
                                "privateIpAddress": "111.111.111.111",
                                "productCodes": {},
                                "rootDeviceName": "/dev/xvda",
                                "rootDeviceType": "ebs",
                                "sourceDestCheck": True,
                                "stateReason": {"code": "pending", "message": "pending"},
                                "subnetId": "subnet-0ae6d533cb0b18193",
                                "tagSet": {"items": [{"key": "Name", "value": "test2"}]},
                                "virtualizationType": "hvm",
                                "vpcId": "vpc-0f59e8f1222b0de6a",
                            },
                        ],
                    },
                    "ownerId": "123123123123",
                    "requestId": "25c95576-a825-44fd-971d-5a52c1e3b2be",
                    "reservationId": "r-02debcf2c4878bc7f",
                },
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "AWS Internal",
                "userAgent": "AWS Internal",
                "userIdentity": {
                    "accessKeyId": "ASIA5PZQZ5QHAW6RFPO5",
                    "accountId": "123123123123",
                    "arn": "arn:aws:sts::123123123123:assumed-role/DevAdministrator/temp_user",
                    "principalId": "AROA5PZQZ5QHBULW27VAC:temp_user",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-10-20T14:14:22Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123123123123",
                            "arn": "arn:aws:iam::123123123123:role/DevAdministrator",
                            "principalId": "AROA5PZQZ5QHBULW27VAC",
                            "type": "Role",
                            "userName": "DevAdministrator",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="RunInstance - Dry Run ",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "errorCode": "Client.DryRunOperation",
                "errorMessage": "Request would have succeeded, but DryRun flag is set.",
                "eventCategory": "Management",
                "eventID": "ab804e72-7237-49c6-8f20-c3ef09859e78",
                "eventName": "RunInstances",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2022-10-13 16:35:33",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_any_aws_account_ids": ["123123123123"],
                "p_any_aws_arns": [
                    "arn:aws:iam::123123123123:role/aws-service-role/eks-nodegroup.amazonaws.com/AWSServiceRoleForAmazonEKSNodegroup",
                    "arn:aws:sts::123123123123:assumed-role/AWSServiceRoleForAmazonEKSNodegroup/EKS",
                ],
                "p_any_domain_names": ["eks-nodegroup.amazonaws.com"],
                "p_any_usernames": ["AWSServiceRoleForAmazonEKSNodegroup"],
                "p_event_time": "2022-10-13 16:35:33",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-10-13 16:42:16.583",
                "p_row_id": "464736612772b0bd8dd8c5fc1384d209",
                "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
                "p_source_label": "Panther Identity Org CloudTrail",
                "readOnly": False,
                "recipientAccountId": "123123123123",
                "requestID": "5e7636b9-d5b5-4bb1-8e37-f98ae7c04e52",
                "requestParameters": {
                    "availabilityZone": "us-west-2b",
                    "blockDeviceMapping": {},
                    "clientToken": "27bec563-7673-4b80-8e00-3537e3b7ad6b",
                    "disableApiStop": False,
                    "disableApiTermination": False,
                    "instanceType": "m5.xlarge",
                    "instancesSet": {"items": [{"maxCount": 1, "minCount": 1}]},
                    "launchTemplate": {"launchTemplateId": "lt-0622a7ff26539376a", "version": "4"},
                    "monitoring": {"enabled": False},
                    "subnetId": "subnet-0d465e7cad854a993",
                },
                "sourceIPAddress": "eks-nodegroup.amazonaws.com",
                "userAgent": "eks-nodegroup.amazonaws.com",
                "userIdentity": {
                    "accountId": "123123123123",
                    "arn": "arn:aws:sts::123123123123:assumed-role/AWSServiceRoleForAmazonEKSNodegroup/EKS",
                    "invokedBy": "eks-nodegroup.amazonaws.com",
                    "principalId": "AROAZBD2CNPWD5DHZ366F:EKS",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-10-13T16:35:31Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "123123123123",
                            "arn": "arn:aws:iam::123123123123:role/aws-service-role/eks-nodegroup.amazonaws.com/AWSServiceRoleForAmazonEKSNodegroup",
                            "principalId": "AROAZBD2CNPWD5DHZ366F",
                            "type": "Role",
                            "userName": "AWSServiceRoleForAmazonEKSNodegroup",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="CopyImage - UserIdentity Null",
            expected_result=True,
            log={
                "awsRegion": "us-east-1",
                "eventCategory": "Management",
                "eventID": "0ea3f05a-066c-43f9-8869-393ba67e7936",
                "eventName": "CreateImage",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2022-09-29 22:25:17",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "123456789101",
                "requestID": "e686939a-a08a-4fd6-abf5-ffffffffffff",
                "requestParameters": {
                    "blockDeviceMapping": {
                        "items": [{"deviceName": "/dev/xvda", "ebs": {"deleteOnTermination": True, "volumeSize": 8}}],
                    },
                    "instanceId": "i-0381a3817f72a949d",
                    "name": "testimage",
                    "noReboot": False,
                },
                "responseElements": {
                    "imageId": "ami-06aaf5e4b77161786",
                    "requestId": "e686939a-a08a-4fd6-abf5-9ea34793cf25",
                },
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "AWS Internal",
                "userAgent": "AWS Internal",
                "userIdentity": {
                    "accessKeyId": "ASIA5PZQZ5QHE2FUNXHR",
                    "accountId": "123456789101",
                    "arn": "arn:aws:sts::123456789101:assumed-role/DevAdministrator/test_user",
                    "principalId": "AROA5PZQZ5QHBULW27VAC:test_user",
                    "invokedBy": None,
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-09-29T22:22:46Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789101",
                            "arn": "arn:aws:iam::123456789101:role/DevAdministrator",
                            "principalId": "AROA5PZQZ5QHBULW27VAC",
                            "type": "Role",
                            "userName": "DevAdministrator",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteGovernmentBackedAttack(Rule):
    id = "GSuite.GovernmentBackedAttack-prototype"
    display_name = "GSuite Government Backed Attack"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.CRITICAL
    default_description = "GSuite reported that it detected a government backed attack against your account.\n"
    default_reference = "https://support.google.com/a/answer/9007870?hl=en"
    default_runbook = "Followup with GSuite support for more details.\n"
    summary_attributes = ["actor:email"]

    def rule(self, event):
        if event.deep_get("id", "applicationName") != "login":
            return False
        return bool(event.get("name") == "gov_attack_warning")

    def title(self, event):
        return f"User [{event.deep_get('actor', 'email', default='<UNKNOWN_EMAIL>')}] may have been targeted by a government attack"

    tests = [
        RuleTest(
            name="Normal Login Event",
            expected_result=False,
            log={
                "id": {"applicationName": "login"},
                "actor": {"email": "homer.simpson@example.com"},
                "type": "login",
                "name": "login_success",
                "parameters": {"is_suspicious": None, "login_challenge_method": ["none"]},
            },
        ),
        RuleTest(
            name="Government Backed Attack Warning",
            expected_result=True,
            log={
                "id": {"applicationName": "login"},
                "actor": {"email": "homer.simpson@example.com"},
                "type": "login",
                "name": "gov_attack_warning",
                "parameters": {"is_suspicious": None, "login_challenge_method": ["none"]},
            },
        ),
    ]



"""
Panther Analysis Tool is a command line interface for writing,
testing, and packaging policies/rules.
Copyright (C) 2020 Panther Labs Inc

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import datetime
import json
from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Any, Dict, Generic, List, Optional, TypeVar

import dateutil.parser

ResponseData = TypeVar("ResponseData")


class BackendError(Exception):
    permanent: bool = False


class PermanentBackendError(BackendError):
    permanent: bool = True


class UnsupportedEndpointError(Exception):
    pass


@dataclass(frozen=True)
class BackendResponse(Generic[ResponseData]):
    data: ResponseData
    status_code: int


@dataclass(frozen=True)
class BackendCheckResponse:
    success: bool
    message: str


@dataclass(frozen=True)
class BulkUploadPresignedURLParams:
    pypanther_version: str


@dataclass(frozen=True)
class BulkUploadPresignedURLResponse:
    detections_url: str
    session_id: str


@dataclass(frozen=True)
class BulkUploadDetectionsParams:
    session_id: str
    dry_run: bool


@dataclass(frozen=True)
class BulkUploadDetectionsResponse:
    job_id: str


@dataclass(frozen=True)
class BulkUploadDetectionsStatusParams:
    job_id: str


@dataclass
class BulkUploadDetectionsError:
    error: str

    @classmethod
    def from_json(cls, data: str) -> "BulkUploadDetectionsError":
        return BulkUploadDetectionsError.from_dict(json.loads(data))

    @classmethod
    def from_dict(cls, data: Optional[Dict[str, Any]]) -> "BulkUploadDetectionsError":
        if not data:
            return cls(error="")
        err = data.get("errorMessage") or ""
        return cls(error=err)

    def asdict(self) -> dict[str, Any]:
        return {
            "error": self.error,
        }


@dataclass(frozen=True)
class BulkUploadDetectionsResults:
    new_rule_ids: list[str]
    modified_rule_ids: list[str]
    deleted_rule_ids: list[str]
    total_rule_ids: list[str]


@dataclass(frozen=True)
class BulkUploadDetectionsStatusResponse:
    message: str
    status: str
    results: BulkUploadDetectionsResults | None


@dataclass(frozen=True)
class ListSchemasParams:
    is_managed: bool


@dataclass(frozen=True)
class UpdateSchemaParams:
    description: str
    name: str
    reference_url: str
    revision: int
    spec: str
    field_discovery_enabled: bool


# pylint: disable=too-many-instance-attributes
@dataclass()
class Schema:
    created_at: str
    description: str
    is_managed: bool
    name: str
    reference_url: str
    revision: int
    spec: str
    updated_at: str
    field_discovery_enabled: bool


@dataclass(frozen=True)
class ListSchemasResponse:
    schemas: List[Schema]


@dataclass(frozen=True)
class UpdateSchemaResponse:
    schema: Schema


class Client(ABC):
    @abstractmethod
    def check(self) -> BackendCheckResponse:
        pass

    @abstractmethod
    def bulk_upload_presigned_url(
        self,
        params: BulkUploadPresignedURLParams,
    ) -> BackendResponse[BulkUploadPresignedURLResponse]:
        pass

    @abstractmethod
    def bulk_upload_detections(
        self,
        params: BulkUploadDetectionsParams,
    ) -> BackendResponse[BulkUploadDetectionsResponse]:
        pass

    @abstractmethod
    def bulk_upload_detections_status(
        self,
        params: BulkUploadDetectionsStatusParams,
    ) -> BackendResponse[BulkUploadDetectionsStatusResponse]:
        pass

    @abstractmethod
    def list_schemas(self, params: ListSchemasParams) -> BackendResponse[ListSchemasResponse]:
        pass

    @abstractmethod
    def update_schema(self, params: UpdateSchemaParams) -> BackendResponse[Any]:
        pass


def backend_response_failed(resp: BackendResponse) -> bool:
    return resp.status_code >= 400 or resp.data.get("statusCode", 0) >= 400


def parse_optional_time(time: Optional[str]) -> Optional[datetime.datetime]:
    return None if time is None else dateutil.parser.parse(time)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSCloudTrailVPCESensitiveAPICalls(Rule):
    id = "AWS.CloudTrail.VPCE.SensitiveAPICalls-prototype"
    display_name = "Sensitive API Calls Via VPC Endpoint"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.MEDIUM
    tags = [
        "AWS",
        "VPC",
        "CloudTrail",
        "Network Boundary Bridging",
        "Cloud Service Discovery",
        "Account Manipulation",
        "Impair Defenses",
    ]
    reports = {"MITRE ATT&CK": ["TA0007:T1526", "TA0003:T1098", "TA0005:T1562", "TA0005:T1599"]}
    default_description = "Detects sensitive or unusual API calls that might indicate lateral movement, reconnaissance, or other malicious activities through VPC Endpoints. Only available for CloudTrail, EC2, KMS, S3, and Secrets Manager services."
    default_runbook = "1. Identify the principal making the sensitive API call and the specific service affected 2. Determine if this action is expected from this principal 3. Check if the API call is one that typically requires additional scrutiny (e.g., logging configuration changes) 4. Investigate whether the VPC Endpoint is configured to properly restrict access 5. Review additional API calls from the same principal for suspicious patterns 6. If unexpected activity is confirmed, consider temporarily restricting the principal's access 7. Document findings and take appropriate remediation steps based on investigation\n"
    default_reference = "https://www.wiz.io/blog/aws-vpc-endpoint-cloudtrail"
    summary_attributes = [
        "userIdentity.principalId",
        "userIdentity.accountId",
        "sourceIPAddress",
        "eventSource",
        "eventName",
    ]
    # Define sensitive API calls to monitor as a constant
    SENSITIVE_APIS = {
        "ec2.amazonaws.com": [
            "DescribeInstances",
            "DescribeNetworkInterfaces",
            "CreateKeyPair",
            "ImportKeyPair",
            "RunInstances",
        ],
        "kms.amazonaws.com": ["Decrypt", "GenerateDataKey", "CreateKey", "ScheduleKeyDeletion"],
        "secretsmanager.amazonaws.com": ["GetSecretValue", "CreateSecret", "PutSecretValue", "DeleteSecret"],
        "s3.amazonaws.com": ["ListAllMyBuckets", "DeleteBucketPolicy", "PutBucketPolicy"],
        "cloudtrail.amazonaws.com": ["StopLogging", "DeleteTrail", "UpdateTrail"],
    }

    def rule(self, event):
        # Check if this is a VPC Endpoint network activity event
        if event.get("eventType") != "AwsVpceEvent" or event.get("eventCategory") != "NetworkActivity":
            return False
        event_source = event.get("eventSource")
        event_name = event.get("eventName")
        if event_source in self.SENSITIVE_APIS and event_name in self.SENSITIVE_APIS[event_source]:
            return True
        return False

    def title(self, event):
        # Use UDM actor_user which leverages the get_actor_user helper function
        # This properly handles various identity types including AssumedRole, Root, etc.
        actor_user = event.udm("actor_user")
        api_name = event.get("eventName", "unknown")
        service = event.get("eventSource", "unknown").split(".")[0]
        return f"Sensitive AWS API [{api_name}] called via VPC Endpoint by [{actor_user}] to service [{service}]"

    def alert_context(self, event):
        account_id = event.deep_get("userIdentity", "accountId", default="")
        context = aws_rule_context(event)
        context.update(
            {
                "account_id": account_id,
                "principal_id": event.deep_get("userIdentity", "principalId", default="unknown"),
                "principal_type": event.deep_get("userIdentity", "type", default="unknown"),
                "actor_user": event.udm("actor_user"),
                "source_ip": event.get("sourceIPAddress", "unknown"),
                "event_source": event.get("eventSource", "unknown"),
                "api_call": event.get("eventName", "unknown"),
                "resources": event.get("resources", []),
                "request_parameters": event.get("requestParameters", {}),
            },
        )
        return context

    tests = [
        RuleTest(
            name="CloudTrail API Call Via VPC Endpoint",
            expected_result=True,
            log={
                "eventVersion": "1.08",
                "eventCategory": "NetworkActivity",
                "eventType": "AwsVpceEvent",
                "eventTime": "2023-03-01T00:00:00Z",
                "awsRegion": "us-east-1",
                "eventSource": "cloudtrail.amazonaws.com",
                "eventName": "UpdateTrail",
                "sourceIPAddress": "10.0.0.1",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AROAEXAMPLE:session-name",
                    "accountId": "111111111111",
                },
                "requestParameters": {"name": "management-events", "isMultiRegionTrail": False},
                "responseElements": None,
                "vpcEndpointId": "vpce-1234abcd",
                "vpcEndpointAccountId": "111111111111",
                "recipientAccountId": "111111111111",
            },
        ),
        RuleTest(
            name="Regular S3 API Call (Not Sensitive)",
            expected_result=False,
            log={
                "eventVersion": "1.08",
                "eventCategory": "NetworkActivity",
                "eventType": "AwsVpceEvent",
                "eventTime": "2023-03-01T00:00:00Z",
                "awsRegion": "us-east-1",
                "eventSource": "s3.amazonaws.com",
                "eventName": "GetObject",
                "sourceIPAddress": "10.0.0.1",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AROAEXAMPLE:session-name",
                    "accountId": "111111111111",
                },
                "requestParameters": {"bucketName": "example-bucket", "key": "example-file.txt"},
                "responseElements": None,
                "vpcEndpointId": "vpce-1234abcd",
                "vpcEndpointAccountId": "111111111111",
                "recipientAccountId": "111111111111",
            },
        ),
        RuleTest(
            name="API Call Without VPC Endpoint",
            expected_result=False,
            log={
                "eventVersion": "1.08",
                "eventCategory": "Management",
                "eventType": "AwsApiCall",
                "eventTime": "2023-03-01T00:00:00Z",
                "awsRegion": "us-east-1",
                "eventSource": "s3.amazonaws.com",
                "eventName": "ListAllMyBuckets",
                "sourceIPAddress": "203.0.113.1",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AROAEXAMPLE:session-name",
                    "accountId": "111111111111",
                },
                "requestParameters": {},
                "responseElements": None,
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.crowdstrike_event_streams import audit_keys_dict, cs_alert_context, str_to_list


@panther_managed
class CrowdstrikeSingleIpAllowlisted(Rule):
    id = "Crowdstrike.SingleIpAllowlisted-prototype"
    display_name = "Crowdstrike Single IP Allowlisted"
    log_types = [LogType.CROWDSTRIKE_EVENT_STREAMS]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0003:T1556.009", "TA0005:T1556.009", "TA0006:T1556.009"]}
    default_description = "A single IP (instead of a CIDR range) was allowlisted. This could indicate a bad actor permitting access from another machine."
    default_runbook = "Validate this action was authorized, and determine the client to which the IP belongs to."

    def get_single_ips(self, event, fieldname="cidrs") -> list[str]:
        """
        Searches the "cidrs" field of the event audit keys, and returns any cidr entries which
        are actually just single IP addresses.
        """
        single_ips = []
        audit_keys = audit_keys_dict(event)
        cidrs = str_to_list(audit_keys.get(fieldname, []))
        for entry in cidrs:
            if "/" not in entry:
                single_ips.append(entry)
            elif entry.endswith("/32"):
                # A 32-bit CIDR range is the same as a single IP address
                single_ips.append(entry[:-3])
        return single_ips

    def rule(self, event):
        # Only alert if an allow list is created or edited
        op_name = event.deep_get("event", "OperationName")
        if op_name not in ("CreateAllowlistGroup", "UpdateAllowlistGroup"):
            return False
        # Only alert if there's a single IP address allowed by the allowlist
        single_ips = self.get_single_ips(event)
        if op_name == "UpdateAllowlistGroup":
            # Remove IPs from single_ips if the weren't recently added
            old_single_ips = set(self.get_single_ips(event, "old_cidrs"))
            single_ips = [ip for ip in single_ips if ip not in old_single_ips]
        # Return true if there were any single IPs
        return len(single_ips) > 0

    def title(self, event):
        # Title format: {actor} granted {contexts_str} access to {a, X} single ip{s}
        single_ips = self.get_single_ips(event)
        actor = event.deep_get("event", "UserId")
        # contexts_str: one of API, UI, or API & UI
        #   Also a more general case: API, UI, and XX (for if they add extra contexts in the future)
        contexts = str_to_list(audit_keys_dict(event).get("contexts", ""))
        if len(contexts) == 0:
            contexts_str = "no contexts"
        elif len(contexts) == 1:
            contexts_str = contexts[0]
        else:
            contexts_str = ", ".join(contexts[:-1]) + " & " + contexts[-1]
        num_ips_str = "a single ip" if len(contexts) == 1 else f"{len(single_ips)} single ips"
        return f"{actor} granted {contexts_str} access to {num_ips_str}"

    def alert_context(self, event):
        context = cs_alert_context(event)
        context.update({"single_ips": self.get_single_ips(event)})
        return context

    tests = [
        RuleTest(
            name="A Single IP In Created Allowlist",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "group_name", "ValueString": "example_group"},
                        {"Key": "description", "ValueString": ""},
                        {"Key": "cidrs", "ValueString": "[1.1.1.1]"},
                        {"Key": "contexts", "ValueString": "[API]"},
                        {"Key": "active", "ValueString": "false"},
                    ],
                    "OperationName": "CreateAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 16:13:13.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_cust_id",
                    "eventCreationTime": "2024-07-26 16:13:13.579000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 365164,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Multiple Single IPs In Created Allowlist",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "group_name", "ValueString": "example_group"},
                        {"Key": "description", "ValueString": ""},
                        {"Key": "cidrs", "ValueString": "[1.1.1.1 2.2.2.2 3.3.3.3/32]"},
                        {"Key": "contexts", "ValueString": "[API UI OTHER]"},
                        {"Key": "active", "ValueString": "false"},
                    ],
                    "OperationName": "CreateAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 16:13:13.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_cust_id",
                    "eventCreationTime": "2024-07-26 16:13:13.579000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 365164,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Single IP Added to existing Allowlist",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "old_group_name", "ValueString": "my_allowlist"},
                        {"Key": "old_cidrs", "ValueString": "[1.2.3.4/8]"},
                        {"Key": "allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "group_name", "ValueString": "my_allowlist"},
                        {"Key": "description", "ValueString": ""},
                        {"Key": "cidrs", "ValueString": "[1.2.3.4/8 32.32.32.32]"},
                        {"Key": "contexts", "ValueString": "[API]"},
                        {"Key": "active", "ValueString": "false"},
                        {"Key": "old_allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "old_description", "ValueString": ""},
                        {"Key": "old_contexts", "ValueString": "[API]"},
                        {"Key": "old_active", "ValueString": "false"},
                    ],
                    "OperationName": "UpdateAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 19:47:16.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_customer_id",
                    "eventCreationTime": "2024-07-26 19:47:16.428000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 366148,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Only CIDR Ranges In Created Allowlist",
            expected_result=False,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "group_name", "ValueString": "example_group"},
                        {"Key": "description", "ValueString": ""},
                        {"Key": "cidrs", "ValueString": "[1.1.1.1/12 2.2.2.2/8 3.3.3.3/4]"},
                        {"Key": "contexts", "ValueString": "[API UI OTHER]"},
                        {"Key": "active", "ValueString": "false"},
                    ],
                    "OperationName": "CreateAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 16:13:13.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_cust_id",
                    "eventCreationTime": "2024-07-26 16:13:13.579000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 365164,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Unrelated Event",
            expected_result=False,
            log={
                "metadata": {
                    "customerIDString": "fake_customer_id",
                    "offset": 341329,
                    "eventType": "AuthActivityAuditEvent",
                    "eventCreationTime": "2024-07-22 15:50:16.923000000",
                    "version": "1.0",
                },
                "event": {
                    "UserId": "sharkey@hobbiton.co",
                    "UserIp": "192.0.2.100",
                    "OperationName": "deleteUser",
                    "ServiceName": "CrowdStrike Authentication",
                    "Success": True,
                    "UTCTimestamp": "2024-07-22 15:50:16.923000000",
                    "AuditKeyValues": [{"Key": "target_name", "ValueString": "frodo.baggins@hobbiton.co"}],
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.mongodb import mongodb_alert_context


@panther_managed
class MongoDBAccessAllowedFromAnywhere(Rule):
    default_description = "Atlas only allows client connections to the database deployment from entries in the project's IP access list. This rule detects when 0.0.0.0/0 is added to that list, which allows access from anywhere."
    display_name = "MongoDB access allowed from anywhere"
    log_types = [LogType.MONGODB_PROJECT_EVENT]
    id = "MongoDB.Access.Allowed.From.Anywhere-prototype"
    default_severity = Severity.HIGH
    tags = ["MongoDB", "Persistence", "Remote Services", "Modify Authentication Process - Conditional Access Policies"]
    reports = {"MITRE ATT&CK": ["TA0003:T1556.009", "TA0008:T1021.007"]}
    default_reference = "https://www.mongodb.com/docs/atlas/security/ip-access-list/"
    default_runbook = "Check if this activity was legitimate. If not, delete 0.0.0.0/0 from the list of allowed ips."

    def rule(self, event):
        if (
            event.get("eventTypeName", "") == "NETWORK_PERMISSION_ENTRY_ADDED"
            and event.get("whitelistEntry", "") == "0.0.0.0/0"
        ):
            return True
        return False

    def title(self, event):
        user = event.get("username", "<USER_NOT_FOUND>")
        group_id = event.get("groupId", "<GROUP_NOT_FOUND>")
        return f"MongoDB: [{user}] has allowed access to group [{group_id}] from anywhere"

    def alert_context(self, event):
        context = mongodb_alert_context(event)
        context["groupId"] = event.get("groupId", "<GROUP_NOT_FOUND>")
        return context

    tests = [
        RuleTest(
            name="Allowed access from anywhere",
            expected_result=True,
            log={
                "created": "2024-04-03 11:13:04.000000000",
                "currentValue": {},
                "eventTypeName": "NETWORK_PERMISSION_ENTRY_ADDED",
                "groupId": "some_group_id",
                "id": "123abc",
                "isGlobalAdmin": False,
                "remoteAddress": "1.2.3.4",
                "userId": "123abc",
                "username": "some_user@company.com",
                "whitelistEntry": "0.0.0.0/0",
            },
        ),
        RuleTest(
            name="Allowed access from specific ip",
            expected_result=False,
            log={
                "created": "2024-04-03 11:13:04.000000000",
                "currentValue": {},
                "eventTypeName": "NETWORK_PERMISSION_ENTRY_ADDED",
                "groupId": "some_group_id",
                "id": "123abc",
                "isGlobalAdmin": False,
                "remoteAddress": "1.2.3.4",
                "userId": "123abc",
                "username": "some_user@company.com",
                "whitelistEntry": "1.2.3.4/32",
            },
        ),
    ]



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.snowflake import query_history_alert_context


@panther_managed
class SnowflakeStreamFileDownloaded(Rule):
    id = "Snowflake.Stream.FileDownloaded-prototype"
    display_name = "Snowflake File Downloaded"
    log_types = [LogType.SNOWFLAKE_QUERY_HISTORY]
    default_severity = Severity.INFO
    create_alert = False
    reports = {"MITRE ATT&CK": ["TA0010:T1041"]}
    default_description = "A file was downloaded from a stage."
    default_reference = (
        "https://cloud.google.com/blog/topics/threat-intelligence/unc5537-snowflake-data-theft-extortion/"
    )
    tags = ["Snowflake", "[MITRE] Exfiltration", "[MITRE] Exfiltration Over C2 Channel"]
    PATH_EXPR = re.compile("GET\\s+(?:\\$\\$|')?@([a-zA-Z0-9_\\./]+)(?:\\$\\$|')?\\s", flags=re.I)
    STAGE_EXPR = re.compile("GET\\s+(?:\\$\\$|')?@([a-zA-Z0-9_\\.]+)", flags=re.I)
    PATH = ""
    STAGE = ""

    def rule(self, event):
        # Check these conditions first to avoid running an expensive regex on every log
        if not all(
            (
                event.get("QUERY_TYPE") == "GET_FILES",
                event.get("EXECUTION_STATUS") == "SUCCESS",
                event.get("QUERY_TEXT") != "GET '@~/worksheet_data/metadata' 'file:///'",
            ),
        ):
            # Avoid alerting for fetching worksheets:
            return False
        self.PATH = self.PATH_EXPR.search(event.get("QUERY_TEXT", ""))
        return self.PATH is not None

    def alert_context(self, event):
        self.STAGE = self.STAGE_EXPR.match(event.get("QUERY_TEXT", ""))
        return query_history_alert_context(event) | {
            "path": self.PATH.group(1),
            "stage": None if not self.STAGE else self.STAGE.group(1).lower(),
        }

    tests = [
        RuleTest(
            name="Worksheet File Downloaded",
            expected_result=False,
            log={
                "p_event_time": "2024-10-09 19:38:06.158000000",
                "p_log_type": "Snowflake.QueryHistory",
                "p_source_label": "SF-Ben",
                "EXECUTION_STATUS": "SUCCESS",
                "QUERY_TEXT": "GET '@~/worksheet_data/metadata' 'file:///'",
                "QUERY_TYPE": "GET_FILES",
                "ROLE_NAME": "PUBLIC",
                "USER_NAME": "CLARK_KENT",
            },
        ),
        RuleTest(
            name="Other File Downloaded",
            expected_result=True,
            log={
                "p_event_time": "2024-10-09 19:38:06.158000000",
                "p_log_type": "Snowflake.QueryHistory",
                "p_source_label": "SF-Ben",
                "EXECUTION_STATUS": "SUCCESS",
                "QUERY_TEXT": "GET @PANTHER_LOGS.PUBLIC.data_exfil/DATA.csv 'file:///Users/lex.luthor/Documents'",
                "QUERY_TYPE": "GET_FILES",
                "ROLE_NAME": "PUBLIC",
                "USER_NAME": "LEX_LUTHOR",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import deep_get
from pypanther.helpers.github import github_reference_url, github_webhook_alert_context, is_cross_fork_pr


@panther_managed
class GitHubWebhookPullRequestTargetUsage(Rule):
    id = "GitHub.Webhook.PullRequestTargetUsage-prototype"
    display_name = "GitHub pull_request_target Workflow Usage"
    log_types = [LogType.GITHUB_WEBHOOK]
    reports = {"MITRE ATT&CK": ["TA0001:T1195.002", "TA0002:T1072", "TA0004:T1134"]}
    tags = ["CI/CD", "Workflow", "Privilege Escalation"]
    default_severity = Severity.HIGH
    default_description = "Detects usage of pull_request_target workflows, which run with elevated privileges and can access secrets even when triggered by external contributors from forks. These workflows pose security risks as they run in the context of the target repository rather than the fork, potentially allowing malicious code execution with write access and secrets. Low severity for non-cross-fork PRs.\n"
    default_runbook = "1. Verify the pull_request_target workflow is necessary and properly secured 2. Check that the workflow doesn't build or run untrusted code from the pull request 3. Ensure the workflow follows security best practices:\n   - Uses explicit checkout with trusted refs\n   - Validates inputs and doesn't execute arbitrary code\n   - Has minimal required permissions\n4. Review the workflow file for potential security vulnerabilities 5. Monitor for unusual activity from external contributors 6. Consider if pull_request event would be sufficient instead\n"
    default_reference = (
        "https://docs.github.com/en/actions/using-workflows/events-that-trigger-workflows#pull_request_target"
    )

    def rule(self, event):
        return event.deep_get("workflow_run", "event") == "pull_request_target"

    def title(self, event):
        workflow_name = event.deep_get("workflow_run", "name", default="<UNKNOWN_WORKFLOW>")
        repo_name = deep_get(event, "repository", "full_name", default="<UNKNOWN_REPO>")
        action = event.get("action", "<UNKNOWN_ACTION>")
        if is_cross_fork_pr(event):
            return (
                f"pull_request_target workflow [{workflow_name}] triggered by cross-fork PR in {repo_name} ({action})"
            )
        return f"pull_request_target workflow [{workflow_name}] triggered in {repo_name} ({action})"

    def alert_context(self, event):
        context = github_webhook_alert_context(event)
        workflow_run = event.get("workflow_run", {})
        if workflow_run:
            context["workflow_run"] = {
                "id": workflow_run.get("id"),
                "name": workflow_run.get("name"),
                "event": workflow_run.get("event"),
                "status": workflow_run.get("status"),
                "conclusion": workflow_run.get("conclusion"),
                "html_url": workflow_run.get("html_url"),
            }
        return context

    def reference(self, event):
        if reference_url := github_reference_url(event):
            return reference_url
        return "DEFAULT"

    def severity(self, event):
        if is_cross_fork_pr(event):
            return "DEFAULT"
        return "LOW"

    tests = [
        RuleTest(
            name="Pull request target workflow completed",
            expected_result=True,
            log={
                "action": "completed",
                "workflow_run": {
                    "id": 12345678,
                    "name": "Security Scan",
                    "event": "pull_request_target",
                    "status": "completed",
                    "conclusion": "success",
                    "html_url": "https://github.com/example-org/example-repo/actions/runs/12345678",
                    "head_branch": "feature-branch",
                    "pull_requests": [
                        {
                            "number": 123,
                            "head": {
                                "ref": "feature-branch",
                                "repo": {
                                    "id": 243627255,
                                    "name": "example-repo",
                                    "full_name": "example-org/example-repo",
                                },
                            },
                            "base": {
                                "ref": "main",
                                "repo": {
                                    "id": 243627255,
                                    "name": "example-repo",
                                    "full_name": "example-org/example-repo",
                                },
                            },
                        },
                    ],
                },
                "repository": {"id": 243627255, "full_name": "example-org/example-repo", "private": True},
            },
        ),
        RuleTest(
            name="Cross-fork pull request target workflow",
            expected_result=True,
            log={
                "action": "completed",
                "workflow_run": {
                    "id": 87654321,
                    "name": "Build and Test",
                    "event": "pull_request_target",
                    "status": "completed",
                    "conclusion": "failure",
                    "html_url": "https://github.com/example-org/example-repo/actions/runs/87654321",
                    "head_branch": "malicious-feature",
                    "pull_requests": [
                        {
                            "number": 456,
                            "head": {
                                "ref": "malicious-feature",
                                "repo": {"id": 999999999, "name": "example-repo", "full_name": "attacker/example-repo"},
                            },
                            "base": {
                                "ref": "main",
                                "repo": {
                                    "id": 243627255,
                                    "name": "example-repo",
                                    "full_name": "example-org/example-repo",
                                },
                            },
                        },
                    ],
                },
                "repository": {"id": 243627255, "full_name": "example-org/example-repo", "private": False},
            },
        ),
        RuleTest(
            name="Regular pull request workflow (not target)",
            expected_result=False,
            log={
                "action": "completed",
                "workflow_run": {
                    "id": 11111111,
                    "name": "CI Tests",
                    "event": "pull_request",
                    "status": "completed",
                    "conclusion": "success",
                    "html_url": "https://github.com/example-org/example-repo/actions/runs/11111111",
                    "head_branch": "safe-feature",
                    "pull_requests": [
                        {
                            "number": 789,
                            "head": {
                                "ref": "safe-feature",
                                "repo": {
                                    "id": 243627255,
                                    "name": "example-repo",
                                    "full_name": "example-org/example-repo",
                                },
                            },
                            "base": {
                                "ref": "main",
                                "repo": {
                                    "id": 243627255,
                                    "name": "example-repo",
                                    "full_name": "example-org/example-repo",
                                },
                            },
                        },
                    ],
                },
                "repository": {"id": 243627255, "full_name": "example-org/example-repo", "private": True},
            },
        ),
        RuleTest(
            name="Push workflow (not pull request related)",
            expected_result=False,
            log={
                "action": "completed",
                "workflow_run": {
                    "id": 22222222,
                    "name": "Deploy",
                    "event": "push",
                    "status": "completed",
                    "conclusion": "success",
                    "html_url": "https://github.com/example-org/example-repo/actions/runs/22222222",
                    "head_branch": "main",
                    "pull_requests": [],
                },
                "repository": {"id": 243627255, "full_name": "example-org/example-repo", "private": True},
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.auth0 import auth0_alert_context, is_auth0_config_event


@panther_managed
class Auth0MFAFactorSettingEnabled(Rule):
    default_description = "An Auth0 user enabled an mfa factor in your organization's mfa settings."
    display_name = "Auth0 mfa factor enabled"
    default_runbook = "Assess if this was done by the user for a valid business reason. Be vigilant to re-enable this setting as it's in the best security interest for your organization's security posture."
    default_reference = "https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors"
    default_severity = Severity.INFO
    log_types = [LogType.AUTH0_EVENTS]
    id = "Auth0.MFA.Factor.Setting.Enabled-prototype"

    def rule(self, event):
        description = event.deep_get("data", "description", default="<NO_DESCRIPTION_FOUND>")
        enabled = event.deep_get("data", "details", "response", "body", "enabled")
        return all(
            [
                description == "Update a Multi-factor Authentication Factor",
                enabled is True,
                is_auth0_config_event(event),
            ],
        )

    def title(self, event):
        user = event.deep_get("data", "details", "request", "auth", "user", "email", default="<NO_USER_FOUND>")
        path = event.deep_get("data", "details", "request", "path", default="<NO_PATH_FOUND>")
        p_source_label = event.get("p_source_label", "<NO_P_SOURCE_LABEL_FOUND>")
        return f"Auth0 User [{user}] enabled mfa factor settings for [{path}] in your organization’s tenant [{p_source_label}]."

    def alert_context(self, event):
        return auth0_alert_context(event)

    tests = [
        RuleTest(
            name="MFA Enabled",
            expected_result=True,
            log={
                "data": {
                    "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                    "client_name": "",
                    "date": "2023-05-15 21:55:18.060000000",
                    "description": "Update a Multi-factor Authentication Factor",
                    "details": {
                        "request": {
                            "auth": {
                                "credentials": {
                                    "jti": "949869e066205b5076e6df203fdd7b9b",
                                    "scopes": [
                                        "create:actions",
                                        "create:actions_log_sessions",
                                        "create:authentication_methods",
                                        "create:client_credentials",
                                        "create:client_grants",
                                        "create:clients",
                                        "create:connections",
                                        "create:custom_domains",
                                        "create:email_provider",
                                        "create:email_templates",
                                        "create:guardian_enrollment_tickets",
                                        "create:integrations",
                                        "create:log_streams",
                                        "create:organization_connections",
                                        "create:organization_invitations",
                                        "create:organization_member_roles",
                                        "create:organization_members",
                                        "create:organizations",
                                        "create:requested_scopes",
                                        "create:resource_servers",
                                        "create:roles",
                                        "create:rules",
                                        "create:shields",
                                        "create:signing_keys",
                                        "create:tenant_invitations",
                                        "create:test_email_dispatch",
                                        "create:users",
                                        "delete:actions",
                                        "delete:anomaly_blocks",
                                        "delete:authentication_methods",
                                        "delete:branding",
                                        "delete:client_credentials",
                                        "delete:client_grants",
                                        "delete:clients",
                                        "delete:connections",
                                        "delete:custom_domains",
                                        "delete:device_credentials",
                                        "delete:email_provider",
                                        "delete:email_templates",
                                        "delete:grants",
                                        "delete:guardian_enrollments",
                                        "delete:integrations",
                                        "delete:log_streams",
                                        "delete:organization_connections",
                                        "delete:organization_invitations",
                                        "delete:organization_member_roles",
                                        "delete:organization_members",
                                        "delete:organizations",
                                        "delete:owners",
                                        "delete:requested_scopes",
                                        "delete:resource_servers",
                                        "delete:roles",
                                        "delete:rules",
                                        "delete:rules_configs",
                                        "delete:shields",
                                        "delete:tenant_invitations",
                                        "delete:tenant_members",
                                        "delete:tenants",
                                        "delete:users",
                                        "read:actions",
                                        "read:anomaly_blocks",
                                        "read:attack_protection",
                                        "read:authentication_methods",
                                        "read:branding",
                                        "read:checks",
                                        "read:client_credentials",
                                        "read:client_grants",
                                        "read:client_keys",
                                        "read:clients",
                                        "read:connections",
                                        "read:custom_domains",
                                        "read:device_credentials",
                                        "read:email_provider",
                                        "read:email_templates",
                                        "read:email_triggers",
                                        "read:entity_counts",
                                        "read:grants",
                                        "read:guardian_factors",
                                        "read:insights",
                                        "read:integrations",
                                        "read:log_streams",
                                        "read:logs",
                                        "read:mfa_policies",
                                        "read:organization_connections",
                                        "read:organization_invitations",
                                        "read:organization_member_roles",
                                        "read:organization_members",
                                        "read:organizations",
                                        "read:prompts",
                                        "read:requested_scopes",
                                        "read:resource_servers",
                                        "read:roles",
                                        "read:rules",
                                        "read:rules_configs",
                                        "read:shields",
                                        "read:signing_keys",
                                        "read:stats",
                                        "read:tenant_invitations",
                                        "read:tenant_members",
                                        "read:tenant_settings",
                                        "read:triggers",
                                        "read:users",
                                        "run:checks",
                                        "update:actions",
                                        "update:attack_protection",
                                        "update:authentication_methods",
                                        "update:branding",
                                        "update:client_credentials",
                                        "update:client_grants",
                                        "update:client_keys",
                                        "update:clients",
                                        "update:connections",
                                        "update:custom_domains",
                                        "update:email_provider",
                                        "update:email_templates",
                                        "update:email_triggers",
                                        "update:guardian_factors",
                                        "update:integrations",
                                        "update:log_streams",
                                        "update:mfa_policies",
                                        "update:organization_connections",
                                        "update:organizations",
                                        "update:prompts",
                                        "update:requested_scopes",
                                        "update:resource_servers",
                                        "update:roles",
                                        "update:rules",
                                        "update:rules_configs",
                                        "update:shields",
                                        "update:signing_keys",
                                        "update:tenant_members",
                                        "update:tenant_settings",
                                        "update:triggers",
                                        "update:users",
                                    ],
                                },
                                "strategy": "jwt",
                                "user": {
                                    "email": "user.name@yourcompany.io",
                                    "name": "User Name",
                                    "user_id": "google-oauth2|105261262156475850461",
                                },
                            },
                            "body": {"AfterAuthentication": False},
                            "channel": "https://manage.auth0.com/",
                            "ip": "12.12.12.12",
                            "method": "patch",
                            "path": "/api/v2/risk-assessment/config",
                            "query": {},
                            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                        },
                        "response": {"body": {"Enabled": True}, "statusCode": 200},
                    },
                    "ip": "12.12.12.12",
                    "log_id": "90020230515215719063964000000000000001223372037488829643",
                    "type": "sapi",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    "user_id": "google-oauth2|105261262156475850461",
                },
                "log_id": "90020230515215719063964000000000000001223372037488829643",
                "p_any_ip_addresses": ["12.12.12.12"],
                "p_any_usernames": ["google-oauth2|105261262156475850461"],
                "p_event_time": "2023-05-15 21:55:18.06",
                "p_log_type": "Auth0.Events",
                "p_parse_time": "2023-05-15 21:58:28.629",
                "p_row_id": "42b742dba9d5f3f284d49b971814",
                "p_schema_version": 0,
                "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
                "p_source_label": "Org Tenant Label",
            },
        ),
        RuleTest(
            name="MFA disabled",
            expected_result=False,
            log={
                "data": {
                    "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                    "client_name": "",
                    "date": "2023-05-15 21:55:18.060000000",
                    "description": "Update a Multi-factor Authentication Factor",
                    "details": {
                        "request": {
                            "auth": {
                                "credentials": {
                                    "jti": "949869e066205b5076e6df203fdd7b9b",
                                    "scopes": [
                                        "create:actions",
                                        "create:actions_log_sessions",
                                        "create:authentication_methods",
                                        "create:client_credentials",
                                        "create:client_grants",
                                        "create:clients",
                                        "create:connections",
                                        "create:custom_domains",
                                        "create:email_provider",
                                        "create:email_templates",
                                        "create:guardian_enrollment_tickets",
                                        "create:integrations",
                                        "create:log_streams",
                                        "create:organization_connections",
                                        "create:organization_invitations",
                                        "create:organization_member_roles",
                                        "create:organization_members",
                                        "create:organizations",
                                        "create:requested_scopes",
                                        "create:resource_servers",
                                        "create:roles",
                                        "create:rules",
                                        "create:shields",
                                        "create:signing_keys",
                                        "create:tenant_invitations",
                                        "create:test_email_dispatch",
                                        "create:users",
                                        "delete:actions",
                                        "delete:anomaly_blocks",
                                        "delete:authentication_methods",
                                        "delete:branding",
                                        "delete:client_credentials",
                                        "delete:client_grants",
                                        "delete:clients",
                                        "delete:connections",
                                        "delete:custom_domains",
                                        "delete:device_credentials",
                                        "delete:email_provider",
                                        "delete:email_templates",
                                        "delete:grants",
                                        "delete:guardian_enrollments",
                                        "delete:integrations",
                                        "delete:log_streams",
                                        "delete:organization_connections",
                                        "delete:organization_invitations",
                                        "delete:organization_member_roles",
                                        "delete:organization_members",
                                        "delete:organizations",
                                        "delete:owners",
                                        "delete:requested_scopes",
                                        "delete:resource_servers",
                                        "delete:roles",
                                        "delete:rules",
                                        "delete:rules_configs",
                                        "delete:shields",
                                        "delete:tenant_invitations",
                                        "delete:tenant_members",
                                        "delete:tenants",
                                        "delete:users",
                                        "read:actions",
                                        "read:anomaly_blocks",
                                        "read:attack_protection",
                                        "read:authentication_methods",
                                        "read:branding",
                                        "read:checks",
                                        "read:client_credentials",
                                        "read:client_grants",
                                        "read:client_keys",
                                        "read:clients",
                                        "read:connections",
                                        "read:custom_domains",
                                        "read:device_credentials",
                                        "read:email_provider",
                                        "read:email_templates",
                                        "read:email_triggers",
                                        "read:entity_counts",
                                        "read:grants",
                                        "read:guardian_factors",
                                        "read:insights",
                                        "read:integrations",
                                        "read:log_streams",
                                        "read:logs",
                                        "read:mfa_policies",
                                        "read:organization_connections",
                                        "read:organization_invitations",
                                        "read:organization_member_roles",
                                        "read:organization_members",
                                        "read:organizations",
                                        "read:prompts",
                                        "read:requested_scopes",
                                        "read:resource_servers",
                                        "read:roles",
                                        "read:rules",
                                        "read:rules_configs",
                                        "read:shields",
                                        "read:signing_keys",
                                        "read:stats",
                                        "read:tenant_invitations",
                                        "read:tenant_members",
                                        "read:tenant_settings",
                                        "read:triggers",
                                        "read:users",
                                        "run:checks",
                                        "update:actions",
                                        "update:attack_protection",
                                        "update:authentication_methods",
                                        "update:branding",
                                        "update:client_credentials",
                                        "update:client_grants",
                                        "update:client_keys",
                                        "update:clients",
                                        "update:connections",
                                        "update:custom_domains",
                                        "update:email_provider",
                                        "update:email_templates",
                                        "update:email_triggers",
                                        "update:guardian_factors",
                                        "update:integrations",
                                        "update:log_streams",
                                        "update:mfa_policies",
                                        "update:organization_connections",
                                        "update:organizations",
                                        "update:prompts",
                                        "update:requested_scopes",
                                        "update:resource_servers",
                                        "update:roles",
                                        "update:rules",
                                        "update:rules_configs",
                                        "update:shields",
                                        "update:signing_keys",
                                        "update:tenant_members",
                                        "update:tenant_settings",
                                        "update:triggers",
                                        "update:users",
                                    ],
                                },
                                "strategy": "jwt",
                                "user": {
                                    "email": "user.name@yourcompany.io",
                                    "name": "User Name",
                                    "user_id": "google-oauth2|105261262156475850461",
                                },
                            },
                            "body": {"AfterAuthentication": False},
                            "channel": "https://manage.auth0.com/",
                            "ip": "12.12.12.12",
                            "method": "patch",
                            "path": "/api/v2/risk-assessment/config",
                            "query": {},
                            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                        },
                        "response": {"body": {"Enabled": False}, "statusCode": 200},
                    },
                    "ip": "12.12.12.12",
                    "log_id": "90020230515215719063964000000000000001223372037488829643",
                    "type": "sapi",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    "user_id": "google-oauth2|105261262156475850461",
                },
                "log_id": "90020230515215719063964000000000000001223372037488829643",
                "p_any_ip_addresses": ["12.12.12.12"],
                "p_any_usernames": ["google-oauth2|105261262156475850461"],
                "p_event_time": "2023-05-15 21:55:18.06",
                "p_log_type": "Auth0.Events",
                "p_parse_time": "2023-05-15 21:58:28.629",
                "p_row_id": "42b742dba9d5f3f284d49b971814",
                "p_schema_version": 0,
                "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
                "p_source_label": "Org Tenant Label",
            },
        ),
    ]



"""
Used to define global helpers and variables for GitHub events.
"""

from pypanther.helpers.base import deep_get


def github_alert_context(event):
    return {
        "action": event.get("action", ""),
        "actor": event.get("actor", ""),
        "actor_location": event.deep_get("actor_location", "country_code"),
        "org": event.get("org", ""),
        "repo": event.get("repo", ""),
        "user": event.get("user", ""),
    }


def is_cross_fork_pr(event):
    """
    Check if this is a cross-fork pull request.

    Args:
    ----
        event: GitHub webhook event

    Returns:
    -------
        bool: True if this is a cross-fork PR (head repo != base repo), False otherwise

    """
    # Check direct pull_request event
    head_repo = event.deep_get("pull_request", "head", "repo", "full_name")
    base_repo = event.deep_get("pull_request", "base", "repo", "full_name")

    if head_repo and base_repo:
        return head_repo != base_repo

    # Check workflow_run event with pull_requests array
    pull_requests = event.deep_get("workflow_run", "pull_requests", default=[])
    for pull_request in pull_requests:
        pr_head_repo = deep_get(pull_request, "head", "repo", "id")
        pr_base_repo = deep_get(pull_request, "base", "repo", "id")
        if pr_head_repo and pr_base_repo and pr_head_repo != pr_base_repo:
            return True

    return False


def is_pull_request_event(event):
    """
    Check if this is a pull request related webhook event.

    Args:
    ----
        event: GitHub webhook event

    Returns:
    -------
        bool: True if this is a pull request event, False otherwise

    """
    pr_actions = [
        "opened",
        "synchronize",
        "reopened",
        "closed",
        "assigned",
        "unassigned",
        "labeled",
        "unlabeled",
        "edited",
        "ready_for_review",
        "converted_to_draft",
    ]
    return event.get("action") in pr_actions and event.get("pull_request") is not None


def github_webhook_alert_context(event):
    """
    Enhanced GitHub webhook alert context for GitHub.Webhook log type.
    Provides more detailed context than the basic github_alert_context.

    Args:
    ----
        event: GitHub webhook event

    Returns:
    -------
        dict: Enhanced alert context with repository, sender, pull request, and push info

    """
    context = github_alert_context(event)

    # Add repository details
    repository = event.get("repository", {})
    context["repository"] = {
        "name": repository.get("name"),
        "full_name": repository.get("full_name"),
        "private": repository.get("private"),
        "fork": repository.get("fork"),
        "default_branch": repository.get("default_branch"),
        "html_url": repository.get("html_url"),
    }

    # Add sender details
    sender = event.get("sender", {})
    if sender:
        context["sender"] = {
            "login": sender.get("login"),
            "id": sender.get("id"),
            "type": sender.get("type"),
            "html_url": sender.get("html_url"),
        }

    # Add pull request details if this is a PR event
    if is_pull_request_event(event):
        pull_request = event.get("pull_request", {})
        context["pull_request"] = {
            "number": pull_request.get("number"),
            "title": pull_request.get("title"),
            "state": pull_request.get("state"),
            "user": deep_get(pull_request, "user", "login"),
            "draft": pull_request.get("draft"),
            "html_url": pull_request.get("html_url"),
            "created_at": pull_request.get("created_at"),
            "updated_at": pull_request.get("updated_at"),
        }

        # Add fork analysis for PR events
        context["fork_analysis"] = {
            "is_cross_fork": is_cross_fork_pr(event),
            "head_repo": deep_get(pull_request, "head", "repo", "full_name"),
            "base_repo": deep_get(pull_request, "base", "repo", "full_name"),
            "head_ref": deep_get(pull_request, "head", "ref"),
            "base_ref": deep_get(pull_request, "base", "ref"),
            "head_sha": deep_get(pull_request, "head", "sha"),
            "base_sha": deep_get(pull_request, "base", "sha"),
        }

    # Add push details if this is a push event
    pusher = event.get("pusher")
    if pusher:
        context["push_details"] = {
            "pusher": pusher,
            "ref": event.get("ref"),
            "before": event.get("before"),
            "after": event.get("after"),
            "forced": event.get("forced"),
            "size": len(event.get("commits", [])),
        }

        # Add head commit details for push events
        head_commit = event.get("head_commit", {})
        if head_commit:
            context["head_commit"] = {
                "id": head_commit.get("id"),
                "message": head_commit.get("message"),
                "author": head_commit.get("author", {}).get("name"),
                "committer": head_commit.get("committer", {}).get("name"),
                "timestamp": head_commit.get("timestamp"),
                "url": head_commit.get("url"),
            }

    return context


def github_reference_url(event):
    """
    Generate a reference URL for GitHub webhook events to link back to the source.

    Args:
    ----
        event: GitHub webhook event

    Returns:
    -------
        str: URL to the PR, commit, or repository depending on event type, or None if unavailable

    """
    # For pull request events, link to the PR
    if is_pull_request_event(event):
        pr_url = deep_get(event, "pull_request", "html_url")
        if pr_url:
            return pr_url

    # For push events, link to the compare view
    compare_url = event.get("compare")
    if compare_url:
        return compare_url

    # Fallback to workflow run URL
    workflow_run_url = deep_get(event, "workflow_run", "html_url")
    if workflow_run_url:
        return workflow_run_url

    # Fallback to repository URL
    repo_url = deep_get(event, "repository", "html_url")
    if repo_url:
        return repo_url

    return None



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPCloudfunctionsFunctionsUpdate(Rule):
    id = "GCP.Cloudfunctions.Functions.Update-prototype"
    display_name = "GCP cloudfunctions functions update"
    default_description = "The Identity and Access Management (IAM) service manages authorization and authentication for a GCP environment. This means that there are very likely multiple privilege escalation methods that use the IAM service and/or its permissions."
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.HIGH
    default_reference = "https://rhinosecuritylabs.com/gcp/privilege-escalation-google-cloud-platform-part-1/"
    default_runbook = "Confirm this was authorized and necessary behavior. This is not a vulnerability in GCP, it is a vulnerability in how GCP environment is configured, so it is necessary to be aware of these attack vectors and to defend against them. It’s also important to remember that privilege escalation does not necessarily need to pass through the IAM service to be effective. Make sure to follow the principle of least-privilege in your environments to help mitigate these security risks."
    reports = {"MITRE ATT&CK": ["TA0004:T1548"]}

    def rule(self, event):
        authorization_info = event.deep_walk("protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if auth.get("permission") == "cloudfunctions.functions.update" and auth.get("granted") is True:
                return True
        return False

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        operation = event.deep_get("protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project_id = event.deep_get("resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] performed [{operation}] on project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)

    tests = [
        RuleTest(
            name="privilege-escalation",
            expected_result=True,
            log={
                "protoPayload": {
                    "authorizationInfo": [{"granted": True, "permission": "cloudfunctions.functions.update"}],
                    "methodName": "v2.deploymentmanager.deployments.insert",
                    "serviceName": "deploymentmanager.googleapis.com",
                },
                "receiveTimestamp": "2024-01-19 13:47:19.465856238",
                "resource": {
                    "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                    "type": "deployment",
                },
                "severity": "NOTICE",
                "timestamp": "2024-01-19 13:47:18.279921000",
            },
        ),
        RuleTest(
            name="fail",
            expected_result=False,
            log={
                "protoPayload": {
                    "authorizationInfo": [{"granted": False, "permission": "cloudfunctions.functions.update"}],
                    "methodName": "v2.deploymentmanager.deployments.insert",
                    "serviceName": "deploymentmanager.googleapis.com",
                },
                "receiveTimestamp": "2024-01-19 13:47:19.465856238",
                "resource": {
                    "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                    "type": "deployment",
                },
                "severity": "NOTICE",
                "timestamp": "2024-01-19 13:47:18.279921000",
            },
        ),
    ]



from pypanther.rules.gitlab.gitlab_audit_password_reset_multiple_emails import (
    GitLabAuditPasswordResetMultipleEmails as GitLabAuditPasswordResetMultipleEmails,
)
from pypanther.rules.gitlab.gitlab_production_password_reset_multiple_emails import (
    GitLabProductionPasswordResetMultipleEmails as GitLabProductionPasswordResetMultipleEmails,
)
